@@ -21,12 +21,7 @@ EXCLUDE = (
     "cmd/gvedit/csettings.h",
     "cmd/gvedit/imageviewer.cpp",
     "cmd/gvedit/imageviewer.h",
-    "cmd/gvedit/main.cpp",
-    "cmd/gvedit/mainwindow.cpp",
-    "cmd/gvedit/mainwindow.h",
-    "cmd/gvedit/mdichild.cpp",
     "cmd/gvedit/mdichild.h",
-    "cmd/gvedit/ui_settings.h",
     "cmd/gvmap/cluster.c",
     "cmd/gvmap/country_graph_coloring.c",
     "cmd/gvmap/country_graph_coloring.h",
@@ -50,21 +45,15 @@ EXCLUDE = (
     "cmd/smyrna/gui/appmouse.c",
     "cmd/smyrna/gui/appmouse.h",
     "cmd/smyrna/gui/callbacks.c",
-    "cmd/smyrna/gui/callbacks.h",
     "cmd/smyrna/gui/datalistcallbacks.c",
-    "cmd/smyrna/gui/datalistcallbacks.h",
     "cmd/smyrna/gui/frmobjectui.c",
-    "cmd/smyrna/gui/frmobjectui.h",
     "cmd/smyrna/gui/glcompui.c",
     "cmd/smyrna/gui/glcompui.h",
     "cmd/smyrna/gui/gui.c",
     "cmd/smyrna/gui/gui.h",
     "cmd/smyrna/gui/menucallbacks.c",
-    "cmd/smyrna/gui/menucallbacks.h",
     "cmd/smyrna/gui/toolboxcallbacks.c",
-    "cmd/smyrna/gui/toolboxcallbacks.h",
     "cmd/smyrna/gui/topviewsettings.c",
-    "cmd/smyrna/gui/topviewsettings.h",
     "cmd/smyrna/gvprpipe.c",
     "cmd/smyrna/hier.c",
     "cmd/smyrna/hier.h",
@@ -97,10 +86,8 @@ EXCLUDE = (
     "cmd/tools/cvtgxl.c",
     "cmd/tools/dijkstra.c",
     "cmd/tools/gc.c",
-    "cmd/tools/gml2gv.c",
     "cmd/tools/gml2gv.h",
     "cmd/tools/graph_generator.c",
-    "cmd/tools/graph_generator.h",
     "cmd/tools/graphml2gv.c",
     "cmd/tools/gv2gml.c",
     "cmd/tools/gv2gxl.c",
@@ -126,7 +113,6 @@ EXCLUDE = (
     "lib/ast/chrtoi.c",
     "lib/ast/error.c",
     "lib/ast/error.h",
-    "lib/ast/fmtbuf.c",
     "lib/ast/fmtesc.c",
     "lib/ast/stresc.c",
     "lib/ast/strmatch.c",
@@ -148,11 +134,9 @@ EXCLUDE = (
     "lib/cdt/dttree.c",
     "lib/cdt/dtview.c",
     "lib/cdt/dtwalk.c",
-    "lib/cgraph/agerror.c",
     "lib/cgraph/apply.c",
     "lib/cgraph/attr.c",
     "lib/cgraph/cghdr.h",
-    "lib/cgraph/cgraph.h",
     "lib/cgraph/edge.c",
     "lib/cgraph/graph.c",
     "lib/cgraph/id.c",
@@ -193,7 +177,6 @@ EXCLUDE = (
     "lib/common/const.h",
     "lib/common/ellipse.c",
     "lib/common/emit.c",
-    "lib/common/entities.h",
     "lib/common/geom.c",
     "lib/common/geom.h",
     "lib/common/geomprocs.h",
@@ -204,12 +187,8 @@ EXCLUDE = (
     "lib/common/htmltable.c",
     "lib/common/htmltable.h",
     "lib/common/input.c",
-    "lib/common/intset.c",
-    "lib/common/intset.h",
     "lib/common/labels.c",
     "lib/common/macros.h",
-    "lib/common/memory.c",
-    "lib/common/memory.h",
     "lib/common/ns.c",
     "lib/common/output.c",
     "lib/common/pointset.c",
@@ -239,7 +218,6 @@ EXCLUDE = (
     "lib/dotgen/dot.h",
     "lib/dotgen/dotinit.c",
     "lib/dotgen/dotprocs.h",
-    "lib/dotgen/dotsplines.c",
     "lib/dotgen/fastgr.c",
     "lib/dotgen/flat.c",
     "lib/dotgen/mincross.c",
@@ -292,19 +270,15 @@ EXCLUDE = (
     "lib/glcomp/glcompimage.c",
     "lib/glcomp/glcompimage.h",
     "lib/glcomp/glcomplabel.c",
-    "lib/glcomp/glcomplabel.h",
     "lib/glcomp/glcompmouse.c",
     "lib/glcomp/glcompmouse.h",
     "lib/glcomp/glcomppanel.c",
-    "lib/glcomp/glcomppanel.h",
     "lib/glcomp/glcompset.c",
     "lib/glcomp/glcompset.h",
     "lib/glcomp/glcomptextpng.c",
-    "lib/glcomp/glcomptextpng.h",
     "lib/glcomp/glcomptexture.c",
     "lib/glcomp/glcomptexture.h",
     "lib/glcomp/glpangofont.c",
-    "lib/glcomp/glpangofont.h",
     "lib/glcomp/glutils.c",
     "lib/glcomp/glutils.h",
     "lib/gvc/gvc.c",
@@ -312,7 +286,6 @@ EXCLUDE = (
     "lib/gvc/gvcext.h",
     "lib/gvc/gvcint.h",
     "lib/gvc/gvcjob.h",
-    "lib/gvc/gvcommon.h",
     "lib/gvc/gvconfig.c",
     "lib/gvc/gvconfig.h",
     "lib/gvc/gvcontext.c",
@@ -335,17 +308,12 @@ EXCLUDE = (
     "lib/gvc/gvtool_tred.c",
     "lib/gvc/gvusershape.c",
     "lib/gvpr/actions.h",
-    "lib/gvpr/compile.c",
     "lib/gvpr/compile.h",
     "lib/gvpr/gdefs.h",
     "lib/gvpr/gprstate.c",
     "lib/gvpr/gprstate.h",
-    "lib/gvpr/gvpr.c",
     "lib/gvpr/gvpr.h",
-    "lib/gvpr/parse.c",
     "lib/gvpr/parse.h",
-    "lib/gvpr/queue.c",
-    "lib/gvpr/queue.h",
     "lib/gvpr/trie.c",
     "lib/gvpr/trieFA.h",
     "lib/label/index.c",
@@ -394,11 +362,9 @@ EXCLUDE = (
     "lib/neatogen/geometry.c",
     "lib/neatogen/geometry.h",
     "lib/neatogen/heap.c",
-    "lib/neatogen/heap.h",
     "lib/neatogen/hedges.c",
     "lib/neatogen/hedges.h",
     "lib/neatogen/info.c",
-    "lib/neatogen/info.h",
     "lib/neatogen/kkutils.c",
     "lib/neatogen/kkutils.h",
     "lib/neatogen/legal.c",
@@ -421,7 +387,6 @@ EXCLUDE = (
     "lib/neatogen/pca.h",
     "lib/neatogen/poly.c",
     "lib/neatogen/poly.h",
-    "lib/neatogen/printvis.c",
     "lib/neatogen/quad_prog_solve.c",
     "lib/neatogen/quad_prog_solver.h",
     "lib/neatogen/quad_prog_vpsc.c",
@@ -433,19 +398,16 @@ EXCLUDE = (
     "lib/neatogen/site.c",
     "lib/neatogen/site.h",
     "lib/neatogen/smart_ini_x.c",
-    "lib/neatogen/solve.c",
     "lib/neatogen/sparsegraph.h",
     "lib/neatogen/stress.c",
     "lib/neatogen/stress.h",
     "lib/neatogen/stuff.c",
     "lib/neatogen/voronoi.c",
-    "lib/neatogen/voronoi.h",
     "lib/ortho/fPQ.c",
     "lib/ortho/fPQ.h",
     "lib/ortho/maze.c",
     "lib/ortho/maze.h",
     "lib/ortho/ortho.c",
-    "lib/ortho/ortho.h",
     "lib/ortho/partition.c",
     "lib/ortho/rawgraph.c",
     "lib/ortho/rawgraph.h",
@@ -456,8 +418,6 @@ EXCLUDE = (
     "lib/ortho/trapezoid.c",
     "lib/osage/osage.h",
     "lib/osage/osageinit.c",
-    "lib/pack/ccomps.c",
-    "lib/pack/pack.c",
     "lib/pack/pack.h",
     "lib/patchwork/patchwork.c",
     "lib/patchwork/patchwork.h",
@@ -465,10 +425,8 @@ EXCLUDE = (
     "lib/patchwork/tree_map.c",
     "lib/patchwork/tree_map.h",
     "lib/pathplan/cvt.c",
-    "lib/pathplan/inpoly.c",
     "lib/pathplan/pathgeom.h",
     "lib/pathplan/pathplan.h",
-    "lib/pathplan/pathutil.h",
     "lib/pathplan/route.c",
     "lib/pathplan/shortest.c",
     "lib/pathplan/shortestpth.c",
@@ -487,7 +445,6 @@ EXCLUDE = (
     "lib/sfdpgen/post_process.h",
     "lib/sfdpgen/sfdp.h",
     "lib/sfdpgen/sfdpinit.c",
-    "lib/sfdpgen/sfdpinternal.h",
     "lib/sfdpgen/sparse_solve.c",
     "lib/sfdpgen/spring_electrical.c",
     "lib/sfdpgen/spring_electrical.h",
@@ -497,14 +454,11 @@ EXCLUDE = (
     "lib/sfio/sfextern.c",
     "lib/sfio/sfhdr.h",
     "lib/sfio/sfio.h",
-    "lib/sfio/sfscanf.c",
     "lib/sfio/sftable.c",
     "lib/sfio/sfprint.c",
     "lib/sfio/sfvscanf.c",
     "lib/sparse/DotIO.c",
     "lib/sparse/DotIO.h",
-    "lib/sparse/LinkedList.c",
-    "lib/sparse/LinkedList.h",
     "lib/sparse/QuadTree.c",
     "lib/sparse/QuadTree.h",
     "lib/sparse/SparseMatrix.c",
@@ -540,9 +494,6 @@ EXCLUDE = (
     "lib/vpsc/csolve_VPSC.h",
     "lib/vpsc/generate-constraints.cpp",
     "lib/vpsc/generate-constraints.h",
-    "lib/vpsc/pairingheap/PairingHeap.cpp",
-    "lib/vpsc/pairingheap/PairingHeap.h",
-    "lib/vpsc/pairingheap/dsexceptions.h",
     "lib/vpsc/solve_VPSC.cpp",
     "lib/vpsc/solve_VPSC.h",
     "lib/vpsc/variable.cpp",
@@ -573,14 +524,12 @@ EXCLUDE = (
     "plugin/core/gvrender_core_fig.c",
     "plugin/core/gvrender_core_json.c",
     "plugin/core/gvrender_core_map.c",
-    "plugin/core/gvrender_core_mp.c",
     "plugin/core/gvrender_core_pic.c",
     "plugin/core/gvrender_core_pov.c",
     "plugin/core/gvrender_core_ps.c",
     "plugin/core/gvrender_core_svg.c",
     "plugin/core/gvrender_core_tk.c",
     "plugin/devil/gvdevice_devil.c",
-    "plugin/devil/gvplugin_devil.c",
     "plugin/dot_layout/gvlayout_dot_layout.c",
     "plugin/dot_layout/gvplugin_dot_layout.c",
     "plugin/gd/gvdevice_gd.c",
@@ -597,22 +546,8 @@ EXCLUDE = (
     "plugin/gdiplus/gvtextlayout_gdiplus.cpp",
     "plugin/gdk/gvdevice_gdk.c",
     "plugin/gdk/gvloadimage_gdk.c",
-    "plugin/gdk/gvplugin_gdk.c",
     "plugin/gs/gvloadimage_gs.c",
-    "plugin/gs/gvplugin_gs.c",
-    "plugin/gtk/callbacks.c",
-    "plugin/gtk/callbacks.h",
-    "plugin/gtk/gvdevice_gtk.c",
-    "plugin/gtk/gvplugin_gtk.c",
-    "plugin/gtk/interface.c",
-    "plugin/gtk/interface.h",
-    "plugin/gtk/support.c",
-    "plugin/gtk/support.h",
-    "plugin/lasi/gvloadimage_lasi.c",
-    "plugin/lasi/gvplugin_lasi.c",
     "plugin/lasi/gvrender_lasi.cpp",
-    "plugin/neato_layout/gvlayout_neato_layout.c",
-    "plugin/neato_layout/gvplugin_neato_layout.c",
     "plugin/pango/gvgetfontlist.h",
     "plugin/pango/gvgetfontlist_pango.c",
     "plugin/pango/gvloadimage_pango.c",
@@ -620,7 +555,6 @@ EXCLUDE = (
     "plugin/pango/gvrender_pango.c",
     "plugin/pango/gvtextlayout_pango.c",
     "plugin/poppler/gvloadimage_poppler.c",
-    "plugin/poppler/gvplugin_poppler.c",
     "plugin/quartz/GVTextLayout.h",
     "plugin/quartz/gvdevice_quartz.c",
     "plugin/quartz/gvloadimage_quartz.c",
@@ -633,45 +567,24 @@ EXCLUDE = (
     "plugin/webp/gvdevice_webp.c",
     "plugin/webp/gvloadimage_webp.c",
     "plugin/webp/gvplugin_webp.c",
-    "plugin/xlib/gvdevice_xlib.c",
-    "plugin/xlib/gvplugin_xlib.c",
-    "tclpkg/gdtclft/gdtclft.c",
     "tclpkg/gv/gv_builtins.c",
     "tclpkg/gv/gv_dummy_init.c",
     "tclpkg/gv/gv_java_init.c",
     "tclpkg/gv/gv_php_init.c",
     "tclpkg/gv/gv_tcl_init.c",
     "tclpkg/tcldot/no_builtins.c",
-    "tclpkg/tcldot/tcldot-edgecmd.c",
-    "tclpkg/tcldot/tcldot-graphcmd.c",
     "tclpkg/tcldot/tcldot-id.c",
     "tclpkg/tcldot/tcldot-io.c",
-    "tclpkg/tcldot/tcldot-nodecmd.c",
     "tclpkg/tcldot/tcldot-util.c",
     "tclpkg/tcldot/tcldot.h",
-    "tclpkg/tcldot/tcldot_builtins.c",
+    "tclpkg/tcldot/tcldot_builtins.cpp",
     "tclpkg/tclpathplan/find_ints.c",
     "tclpkg/tclpathplan/intersect.c",
     "tclpkg/tclpathplan/makecw.c",
     "tclpkg/tclpathplan/simple.h",
     "tclpkg/tclpathplan/tclpathplan.c",
     "tclpkg/tclpathplan/wrapper.c",
-    "tclpkg/tclstubs/tclInt.h",
     "tclpkg/tclstubs/tclStubLib.c",
-    "windows/gvedit/Application.h",
-    "windows/gvedit/GraphX.cpp",
-    "windows/gvedit/UAbout.cpp",
-    "windows/gvedit/UAbout.h",
-    "windows/gvedit/UEditor.cpp",
-    "windows/gvedit/UEditor.h",
-    "windows/gvedit/UPreProcess.cpp",
-    "windows/gvedit/UPreProcess.h",
-    "windows/gvedit/UPreview.cpp",
-    "windows/gvedit/UPreview.h",
-    "windows/gvedit/USettings.cpp",
-    "windows/gvedit/USettings.h",
-    "windows/gvedit/Umain.cpp",
-    "windows/gvedit/Umain.h",
     "windows/include/unistd/unistd.h",
 )
 
@@ -704,7 +617,6 @@ sources = subprocess.check_output(
 )
 
 for source in sources.split("\x00")[:-1]:
-
     print(f"checking {source}...")
 
     original = Path(source).read_text(encoding="utf-8")
