@@ -8,13 +8,13 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #include "config.h"
-
+#include "../tools/openFile.h"
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -30,10 +30,12 @@
 #include <sfdpgen/post_process.h>
 #include <neatogen/overlap.h>
 #include <sparse/clustering.h>
-#include <ingraphs/ingraphs.h>
+#include <cgraph/ingraphs.h>
 #include <sparse/DotIO.h>
 #include <sparse/colorutil.h>
 #include <sparse/color_palette.h>
+#include <util/startswith.h>
+#include <util/unreachable.h>
 
 typedef struct {
     char* cmd;
@@ -126,18 +128,6 @@ static void usage(char* cmd, int eval)
     graphviz_exit(eval);
 }
 
-static FILE *openFile(const char *name, const char* cmd)
-{
-    FILE *fp;
-
-    fp = fopen(name, "w");
-    if (!fp) {
-	fprintf(stderr, "%s: could not open file %s for writing\n", cmd, name);
-	graphviz_exit(-1);
-    }
-    return (fp);
-}
-
 #define HLPFX "ighlight="
 #define N_HLPFX (sizeof(HLPFX)-1)
 
@@ -199,7 +189,7 @@ init(int argc, char **argv, params_t* pm)
     case 'h':
       if (sscanf(optarg, "%d", &v) > 0) {
         pm->nedgep = MAX(0, v);
-      } else if (!strncmp(optarg, HLPFX, N_HLPFX) &&
+      } else if (startswith(optarg, HLPFX) &&
                  sscanf(optarg + N_HLPFX, "%d", &v) > 0) {
         pm->highlight_cluster = MAX(0, v);
       } else {
@@ -231,7 +221,7 @@ init(int argc, char **argv, params_t* pm)
       pm->plotedges = true;
       break;
     case 'o':
-	  pm->outfile = openFile(optarg, pm->cmd);
+	  pm->outfile = openFile(pm->cmd, optarg, "w");
       break;
     case 'O':
       pm->color_optimize = false;
@@ -299,6 +289,8 @@ init(int argc, char **argv, params_t* pm)
         usage(cmd, 1);
       }
       break;
+    default:
+      UNREACHABLE();
     }
   }
 
@@ -321,10 +313,10 @@ validateCluster (int n, int* grouping, int clust_num)
   return 0;
 }
 
-static void 
-makeMap (SparseMatrix graph, int n, double* x, double* width, int* grouping, 
-  char** labels, float* fsz, float* rgb_r, float* rgb_g, float* rgb_b, params_t* pm, Agraph_t* g )
-{
+/// @return 0 on success
+static int makeMap(SparseMatrix graph, int n, double *x, double *width,
+                   int *grouping, char **labels, float *fsz, float *rgb_r,
+                   float *rgb_g, float *rgb_b, params_t *pm, Agraph_t *g) {
   int dim = pm->dim;
   int i;
   SparseMatrix poly_lines, polys, poly_point_map;
@@ -345,10 +337,15 @@ makeMap (SparseMatrix graph, int n, double* x, double* width, int* grouping,
   if (pm->highlight_cluster) {
     pm->highlight_cluster = validateCluster (n, grouping, pm->highlight_cluster);
   }
-  make_map_from_rectangle_groups(pm->include_OK_points,
-				 n, dim, x, width, grouping, graph, pm->bbox_margin, nrandom, &nart, pm->nedgep, 
-				 pm->shore_depth_tol, &nverts, &x_poly, &poly_lines, 
-				 &polys, &polys_groups, &poly_point_map, &country_graph, pm->highlight_cluster);
+  if (make_map_from_rectangle_groups(pm->include_OK_points, n, dim, x, width,
+                                     grouping, graph, pm->bbox_margin, nrandom,
+                                     &nart, pm->nedgep, pm->shore_depth_tol,
+                                     &nverts, &x_poly, &poly_lines, &polys,
+                                     &polys_groups, &poly_point_map,
+                                     &country_graph, pm->highlight_cluster)
+                                     != 0) {
+    return -1;
+  }
 
   if (Verbose) fprintf(stderr,"nart = %d\n",nart);
   /* compute a good color permutation */
@@ -370,19 +367,19 @@ makeMap (SparseMatrix graph, int n, double* x, double* width, int* grouping,
     for (i = 0; i < improve_contiguity_n; i++){
       improve_contiguity(n, dim, grouping, poly_point_map, x, graph);
       nart = nart0;
-      make_map_from_rectangle_groups(pm->include_OK_points,
+      (void)make_map_from_rectangle_groups(pm->include_OK_points,
 				     n, dim, x, width, grouping, graph, pm->bbox_margin, nrandom, &nart, pm->nedgep, 
 				     pm->shore_depth_tol, &nverts, &x_poly, &poly_lines, 
 				     &polys, &polys_groups, &poly_point_map, &country_graph, pm->highlight_cluster);
     }
     {
-      SparseMatrix D;
-      D = SparseMatrix_get_real_adjacency_matrix_symmetrized(graph);
+      SparseMatrix D = SparseMatrix_get_real_adjacency_matrix_symmetrized(graph);
       remove_overlap(dim, D, x, width, 1000, 5000.,
 		     ELSCHEME_NONE, 0, NULL, NULL, true);
+      SparseMatrix_delete(D);
       
       nart = nart0;
-      make_map_from_rectangle_groups(pm->include_OK_points,
+      (void)make_map_from_rectangle_groups(pm->include_OK_points,
 				     n, dim, x, width, grouping, graph, pm->bbox_margin, nrandom, &nart, pm->nedgep, 
 				     pm->shore_depth_tol, &nverts, &x_poly, &poly_lines, 
 				     &polys, &polys_groups, &poly_point_map, &country_graph, pm->highlight_cluster);
@@ -398,26 +395,31 @@ makeMap (SparseMatrix graph, int n, double* x, double* width, int* grouping,
   SparseMatrix_delete(poly_point_map);
   free(x_poly);
   free(polys_groups);
+  return 0;
 }
 
-
-static void mapFromGraph (Agraph_t* g, params_t* pm)
-{
+/// @return 0 on success
+static int mapFromGraph(Agraph_t *g, params_t *pm) {
     SparseMatrix graph;
   int n;
   double* width = NULL;
   double* x;
   char** labels = NULL;
   int* grouping;
-  float* rgb_r;
-  float* rgb_g;
-  float* rgb_b;
+  float* rgb_r = NULL;
+  float* rgb_g = NULL;
+  float* rgb_b = NULL;
   float* fsz;
 
   initDotIO(g);
   graph = Import_coord_clusters_from_dot(g, pm->maxcluster, pm->dim, &n, &width, &x, &grouping, 
 					   &rgb_r,  &rgb_g,  &rgb_b,  &fsz, &labels, pm->color_scheme, pm->clusterMethod, pm->useClusters);
-  makeMap (graph, n, x, width, grouping, labels, fsz, rgb_r, rgb_g, rgb_b, pm, g);
+  const int rc = makeMap(graph, n, x, width, grouping, labels, fsz, rgb_r,
+                         rgb_g, rgb_b, pm, g);
+  free(rgb_r);
+  free(rgb_g);
+  free(rgb_b);
+  return rc;
 }
 
 int main(int argc, char *argv[])
@@ -432,7 +434,9 @@ int main(int argc, char *argv[])
   newIngraph(&ig, pm.infiles);
   while ((g = nextGraph (&ig)) != 0) {
     if (prevg) agclose (prevg);
-    mapFromGraph (g, &pm);
+    if (mapFromGraph(g, &pm) != 0) {
+      graphviz_exit(EXIT_FAILURE);
+    }
     prevg = g;
   }
 
