@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -18,9 +18,6 @@
 #include <math.h>
 #include <glcomp/glcompfont.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif
 /* DRAWING FUNCTIONS 
  * these are opengl based xdot drawing functions 
  * topview drawings are not here
@@ -28,8 +25,4 @@ extern "C" {
 extern drawfunc_t OpFns[];
 extern void drawCircle(float x, float y, float radius, float zdepth);
 extern void drawBorders(ViewInfo * view);
-extern void draw_selpoly(glCompPoly* selPoly);
-
-#ifdef __cplusplus
-}				/* end extern "C" */
-#endif
+extern void draw_selpoly(glCompPoly_t *selPoly);
