@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -11,6 +11,7 @@
 #include "smyrnadefs.h"
 #include "gvprpipe.h"
 #include <common/const.h>
+#include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
@@ -19,10 +20,10 @@
 #include "draw.h"
 #include "gui.h"
 #include "topviewsettings.h"
+#include <util/agxbuf.h>
 #include <viewport.h>
 
 #include <gvpr/gvpr.h>
-extern GladeXML *xml;		//global libglade vars
 
 static ssize_t outfn(void *sp, const char *buf, size_t nbyte, void *dp)
 {
@@ -37,13 +38,12 @@ static ssize_t outfn(void *sp, const char *buf, size_t nbyte, void *dp)
     return (ssize_t)nbyte;
 }
 
-int run_gvpr(Agraph_t * srcGraph, int argc, char *argv[])
-{
-    int i, rv = 1;
+int run_gvpr(Agraph_t * srcGraph, size_t argc, char *argv[]) {
+    int rv = 1;
     gvpropts opts;
     Agraph_t *gs[2];
     static int count;
-    char buf[SMALLBUF];
+    agxbuf buf = {0};
 
     gs[0] = srcGraph;
     gs[1] = 0;
@@ -53,19 +53,20 @@ int run_gvpr(Agraph_t * srcGraph, int argc, char *argv[])
     opts.err = outfn;
     opts.flags = GV_USE_OUTGRAPH;
 
-    rv = gvpr(argc, argv, &opts);
+    assert(argc <= INT_MAX);
+    rv = gvpr((int)argc, argv, &opts);
 
     if (rv) {			/* error */
 	fprintf(stderr, "Error in gvpr\n");
     } else if (opts.n_outgraphs) 
     {
-	refreshViewport(0);
-	snprintf(buf, sizeof(buf), "<%d>", ++count);
+	refreshViewport();
+	agxbprint(&buf, "<%d>", ++count);
 	if (opts.outgraphs[0] != view->g[view->activeGraph])
-	    add_graph_to_viewport(opts.outgraphs[0], buf);
+	    add_graph_to_viewport(opts.outgraphs[0], agxbuse(&buf));
 	if (opts.n_outgraphs > 1)
 	    fprintf(stderr, "Warning: multiple output graphs-discarded\n");
-	for (i = 1; i < opts.n_outgraphs; i++) {
+	for (size_t i = 1; i < opts.n_outgraphs; i++) {
 	    agclose(opts.outgraphs[i]);
 	}
     } else 
@@ -73,5 +74,6 @@ int run_gvpr(Agraph_t * srcGraph, int argc, char *argv[])
 	updateRecord (srcGraph);
         update_graph_from_settings(srcGraph);
     }
+    agxbfree(&buf);
     return rv;
 }
