@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -12,11 +12,11 @@
 #include "tvnodes.h"
 #include "viewport.h"
 #include "topviewfuncs.h"
-#include <cgraph/alloc.h>
 #include <cgraph/strview.h>
 #include <cgraph/tokenize.h>
 #include <stdbool.h>
 #include <string.h>
+#include <util/alloc.h>
 
 typedef struct {
     GType type;
