@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -12,15 +12,7 @@
 
 #include "gui.h"
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
     int tv_show_all(void);
     int tv_hide_all(void);
     int tv_save_as(int);
     void setup_tree (Agraph_t* g);
-
-#ifdef __cplusplus
-}				/* end extern "C" */
-#endif
