@@ -13,14 +13,13 @@
 #include <assert.h>
 #include "viewport.h"
 #include "draw.h"
-#include <cgraph/alloc.h>
 #include <common/color.h>
 #include <glade/glade.h>
 #include "gui.h"
 #include <limits.h>
 #include "menucallbacks.h"
-#include <stddef.h>
 #include <stdbool.h>
+#include <stdlib.h>
 #include <string.h>
 #include "glcompui.h"
 #include "gltemplate.h"
@@ -29,15 +28,13 @@
 #include "arcball.h"
 #include "hotkeymap.h"
 #include "topviewfuncs.h"
-#include <cgraph/exit.h>
-#include <cgraph/strcasecmp.h>
+#include <util/alloc.h>
+#include <util/exit.h>
+#include <util/strcasecmp.h>
 
-
-static colorschemaset *create_color_theme(int themeid);
+static colorschemaset create_color_theme(int themeid);
 
 ViewInfo *view;
-/* these two global variables should be wrapped in something else */
-GtkMessageDialog *Dlg;
 
 static void clear_viewport(ViewInfo *vi) {
     if (vi->graphCount)
@@ -87,7 +84,6 @@ char *get_attribute_value(char *attr, ViewInfo *vi, Agraph_t *g) {
 
 void set_viewport_settings_from_template(ViewInfo *vi, Agraph_t *g) {
     gvcolor_t cl;
-    char *buf;
     colorxlate(get_attribute_value("bordercolor", vi, g), &cl,
 	       RGBA_DOUBLE);
     vi->borderColor.R = (float)cl.u.RGBA[0];
@@ -99,7 +95,7 @@ void set_viewport_settings_from_template(ViewInfo *vi, Agraph_t *g) {
 
     vi->bdVisible = atoi(get_attribute_value("bordervisible", vi, g));
 
-    buf = get_attribute_value("gridcolor", vi, g);
+    const char *buf = get_attribute_value("gridcolor", vi, g);
     colorxlate(buf, &cl, RGBA_DOUBLE);
     vi->gridColor.R = (float)cl.u.RGBA[0];
     vi->gridColor.G = (float)cl.u.RGBA[1];
@@ -157,7 +153,6 @@ void set_viewport_settings_from_template(ViewInfo *vi, Agraph_t *g) {
     vi->edgelabelcolor.B = (float)cl.u.RGBA[2];
     vi->edgelabelcolor.A =
 	(float) atof(get_attribute_value("defaultedgealpha", vi, g));
-    vi->labelwithdegree = atoi(get_attribute_value("labelwithdegree", vi, g));
     vi->labelnumberofnodes =
 	atof(get_attribute_value("labelnumberofnodes", vi, g));
     vi->labelshownodes = atoi(get_attribute_value("labelshownodes", vi, g));
@@ -184,13 +179,17 @@ static gboolean gl_main_expose(void *data) {
 static void get_data_dir(void)
 {
     free(view->template_file);
-    view->template_file = gv_strdup(smyrnaPath("template.dot"));
+    view->template_file = smyrnaPath("template.dot");
+}
+
+static void clear_color_theme(colorschemaset *cs) {
+  free(cs->s);
+  *cs = (colorschemaset){0};
 }
 
 void init_viewport(ViewInfo *vi) {
     FILE *input_file = NULL;
     FILE *input_file2 = NULL;
-    static char* path;
     get_data_dir();
     input_file = fopen(vi->template_file, "rb");
     if (!input_file) {
@@ -208,18 +207,24 @@ void init_viewport(ViewInfo *vi) {
 		vi->template_file);
 	graphviz_exit(-1);
     }
-    if (!path)
-	path = smyrnaPath("attr_widgets.dot");
-    input_file2 = fopen(path, "rb");
+    {
+	char *path = smyrnaPath("attr_widgets.dot");
+	input_file2 = fopen(path, "rb");
+	free(path);
+    }
     if (!input_file2) {
-	fprintf(stderr,	"default attributes template graph file \"%s\" not found\n",smyrnaPath("attr_widgets.dot"));
+	char *attrwidgets = smyrnaPath("attr_widgets.dot");
+	fprintf(stderr,	"default attributes template graph file \"%s\" not found\n", attrwidgets);
+	free(attrwidgets);
 	graphviz_exit(-1);
 
     }
     vi->systemGraphs.attrs_widgets = agread(input_file2, NULL);
     fclose (input_file2);
     if (!vi->systemGraphs.attrs_widgets) {
-	fprintf(stderr,"could not load default attribute widgets graph file \"%s\"\n",smyrnaPath("attr_widgets.dot"));
+	char *attrwidgets = smyrnaPath("attr_widgets.dot");
+	fprintf(stderr,"could not load default attribute widgets graph file \"%s\"\n", attrwidgets);
+	free(attrwidgets);
 	graphviz_exit(-1);
     }
     //init graphs
@@ -306,13 +311,9 @@ void init_viewport(ViewInfo *vi) {
     vi->active_camera = SIZE_MAX;
     set_viewport_settings_from_template(vi, vi->systemGraphs.def_attrs);
     vi->Topview->Graphdata.GraphFileName = NULL;
-    vi->colschms = NULL;
+    clear_color_theme(&vi->colschms);
     vi->arcball = gv_alloc(sizeof(ArcBall_t));
-    vi->keymap.down=0;
     load_mouse_actions(vi);
-    vi->refresh.color=1;
-    vi->refresh.pos=1;
-    vi->refresh.selection=1;
     if(vi->guiMode!=GUI_FULLSCREEN)
 	vi->guiMode=GUI_WINDOWED;
 
@@ -410,9 +411,6 @@ graphRecord (Agraph_t* g)
 void refreshViewport(void)
 {
     Agraph_t *graph = view->g[view->activeGraph];
-    view->refresh.color=1;
-    view->refresh.pos=1;
-    view->refresh.selection=1;
     load_settings_from_graph();
 
     if(view->guiMode!=GUI_FULLSCREEN)
@@ -549,37 +547,22 @@ static float interpol(float minv, float maxv, float minc, float maxc, float x)
     return (x - minv) * (maxc - minc) / (maxv - minv) + minc;
 }
 
-void getcolorfromschema(colorschemaset * sc, float l, float maxl,
-			glCompColor * c)
-{
+void getcolorfromschema(const colorschemaset sc, float l, float maxl,
+                        glCompColor *c) {
     size_t ind;
     float percl = l / maxl;
 
-    if (sc->smooth) {
-	/* For smooth schemas, s[0].perc = 0, so we start with ind=1 */
-	for (ind = 1; ind + 1 < sc->schemacount; ind++) {
-	    if (percl < sc->s[ind].perc)
-		break;
-	}
-	c->R =
-	    interpol(sc->s[ind - 1].perc, sc->s[ind].perc,
-		     sc->s[ind - 1].c.R, sc->s[ind].c.R, percl);
-	c->G =
-	    interpol(sc->s[ind - 1].perc, sc->s[ind].perc,
-		     sc->s[ind - 1].c.G, sc->s[ind].c.G, percl);
-	c->B =
-	    interpol(sc->s[ind - 1].perc, sc->s[ind].perc,
-		     sc->s[ind - 1].c.B, sc->s[ind].c.B, percl);
-    }
-    else {
-	for (ind = 0; ind + 1 < sc->schemacount; ind++) {
-	    if (percl < sc->s[ind].perc)
-		break;
-	}
-	c->R = sc->s[ind].c.R;
-	c->G = sc->s[ind].c.G;
-	c->B = sc->s[ind].c.B;
+    // For smooth schemas, s[0].perc = 0, so we start with ind=1
+    for (ind = 1; ind + 1 < sc.schemacount; ind++) {
+	if (percl < sc.s[ind].perc)
+	    break;
     }
+    c->R = interpol(sc.s[ind - 1].perc, sc.s[ind].perc, sc.s[ind - 1].c.R,
+                    sc.s[ind].c.R, percl);
+    c->G = interpol(sc.s[ind - 1].perc, sc.s[ind].perc, sc.s[ind - 1].c.G,
+                    sc.s[ind].c.G, percl);
+    c->B = interpol(sc.s[ind - 1].perc, sc.s[ind].perc, sc.s[ind - 1].c.B,
+                    sc.s[ind].c.B, percl);
 
     c->A = 1;
 }
@@ -593,7 +576,6 @@ static void set_color_theme_color(colorschemaset * sc, char **colorstr)
     gvcolor_t cl;
     float av_perc;
 
-    sc->smooth = 1;
     av_perc = 1.0 / (float) (colorcnt-1);
     for (size_t ind = 0; ind < colorcnt; ind++) {
         colorxlate(colorstr[ind], &cl, RGBA_DOUBLE);
@@ -605,12 +587,6 @@ static void set_color_theme_color(colorschemaset * sc, char **colorstr)
     }
 }
 
-static void clear_color_theme(colorschemaset * cs)
-{
-    free(cs->s);
-    free(cs);
-}
-
 static char *deep_blue[] = {
     "#C8CBED", "#9297D3", "#0000FF", "#2C2E41"
 };
@@ -636,20 +612,18 @@ static colordata palette[] = {
 };
 #define NUM_SCHEMES (sizeof(palette)/sizeof(colordata))
 
-static colorschemaset *create_color_theme(int themeid)
-{
+static colorschemaset create_color_theme(int themeid) {
     if (themeid < 0 || (int)NUM_SCHEMES <= themeid) {
 	fprintf (stderr, "colorschemaset: illegal themeid %d\n", themeid);
 	return view->colschms;
     }
 
-    colorschemaset *s = gv_alloc(sizeof(colorschemaset));
-    if (view->colschms)
-	clear_color_theme(view->colschms);
+    colorschemaset s = {0};
+    clear_color_theme(&view->colschms);
 
-    s->schemacount = palette[themeid].cnt;
-    s->s = gv_calloc(s->schemacount, sizeof(colorschema));
-    set_color_theme_color(s, palette[themeid].colors);
+    s.schemacount = palette[themeid].cnt;
+    s.s = gv_calloc(s.schemacount, sizeof(colorschema));
+    set_color_theme_color(&s, palette[themeid].colors);
 
     return s;
 }
