@@ -8,21 +8,21 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-#include <cgraph/agxbuf.h>
-#include <cgraph/alloc.h>
-#include <cgraph/startswith.h>
+#include <cgraph/gv_ctype.h>
 #include <common/types.h>
 #include <common/utils.h>
 #include "convert.h"
-#include <ctype.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <util/agxbuf.h>
+#include <util/alloc.h>
+#include <util/startswith.h>
 
 #define EMPTY(s)	((s == 0) || (*s == '\0'))
 #define SLEN(s)     (sizeof(s)-1)
@@ -53,8 +53,9 @@ typedef struct {
     char *unique_name;
 } namev_t;
 
-static namev_t *make_nitem(namev_t *objp, Dtdisc_t *disc) {
+static void *make_nitem(void *p, Dtdisc_t *disc) {
     (void)disc;
+    namev_t *objp = p;
 
     namev_t *np = gv_alloc(sizeof(*np));
     np->name = objp->name;
@@ -62,19 +63,12 @@ static namev_t *make_nitem(namev_t *objp, Dtdisc_t *disc) {
     return np;
 }
 
-static void free_nitem(namev_t *np, Dtdisc_t *disc) {
-    (void)disc;
-
-    free(np);
-}
-
 static Dtdisc_t nameDisc = {
     offsetof(namev_t, name),
     -1,
     offsetof(namev_t, link),
-    (Dtmake_f) make_nitem,
-    (Dtfree_f) free_nitem,
-    NULL,
+    make_nitem,
+    free,
     NULL,
 };
 
@@ -83,9 +77,8 @@ typedef struct {
     char *name;
 } idv_t;
 
-static void free_iditem(idv_t *idp, Dtdisc_t *disc) {
-    (void)disc;
-
+static void free_iditem(void *idv) {
+    idv_t *idp = idv;
     free(idp->name);
     free(idp);
 }
@@ -95,8 +88,7 @@ static Dtdisc_t idDisc = {
     -1,
     offsetof(idv_t, link),
     NULL,
-    (Dtfree_f) free_iditem,
-    NULL,
+    free_iditem,
     NULL,
 };
 
@@ -127,10 +119,10 @@ static void tabover(FILE * gxlFile)
  */
 static bool legalGXLName(const char *id) {
     char c = *id++;
-    if (!isalpha(c) && c != '_' && c != ':')
+    if (!gv_isalpha(c) && c != '_' && c != ':')
 	return false;
     while ((c = *id++)) {
-	if (!isalnum(c) && c != '_' && c != ':' && c != '-' && c != '.')
+	if (!gv_isalnum(c) && c != '_' && c != ':' && c != '-' && c != '.')
 	    return false;
     }
     return true;
@@ -488,7 +480,7 @@ static void writeSubgs(gxlstate_t * stp, Agraph_t * g, FILE * gxlFile)
     }
 }
 
-static bool writeEdgeName(Agedge_t *e, FILE *gxlFile) {
+static void writeEdgeName(Agedge_t *e, FILE *gxlFile) {
     char *p = agnameof(e);
     if (!(EMPTY(p))) {
 	tabover(gxlFile);
@@ -499,20 +491,15 @@ static bool writeEdgeName(Agedge_t *e, FILE *gxlFile) {
 	fprintf(gxlFile, "</string>\n");
 	tabover(gxlFile);
 	fprintf(gxlFile, "\t</attr>\n");
-	return true;
     }
-    return false;
 }
 
 
 static void
 writeNondefaultAttr(void *obj, FILE * gxlFile, Dict_t * defdict)
 {
-    int cnt = 0;
-
     if (AGTYPE(obj) == AGINEDGE || AGTYPE(obj) == AGOUTEDGE) {
-	if (writeEdgeName(obj, gxlFile))
-	    cnt++;
+	writeEdgeName(obj, gxlFile);
     }
     Agattr_t *data = agattrrec(obj);
     if (data) {
