@@ -1,4 +1,5 @@
-#include	"dthdr.h"
+#include	<cdt/dthdr.h>
+#include	<stddef.h>
 
 /*	Flatten a dictionary into a linked list.
 **	This may be used when many traversals are likely.
@@ -8,14 +9,14 @@
 
 Dtlink_t* dtflatten(Dt_t* dt)
 {
-	reg Dtlink_t	*t, *r, *list, *last, **s, **ends;
+	Dtlink_t	*t, *r, *list, *last, **s, **ends;
 
 	/* already flattened */
 	if(dt->data->type&DT_FLATTEN )
 		return dt->data->here;
 
-	list = last = NIL(Dtlink_t*);
-	if(dt->data->type&(DT_SET|DT_BAG))
+	list = last = NULL;
+	if(dt->data->type&DT_SET)
 	{	for(ends = (s = dt->data->htab) + dt->data->ntab; s < ends; ++s)
 		{	if((t = *s) )
 			{	if(last)
@@ -27,7 +28,7 @@ Dtlink_t* dtflatten(Dt_t* dt)
 			}
 		}
 	}
-	else if(dt->data->type&(DT_LIST|DT_STACK|DT_QUEUE) )
+	else if(dt->data->type&DT_QUEUE)
 		list = dt->data->head;
 	else if((r = dt->data->here) ) /*if(dt->data->type&(DT_OSET|DT_OBAG))*/
 	{	while((t = r->left) )
