@@ -1,15 +1,25 @@
+/**
+ * @file
+ * @ingroup cgraph_core
+ * @ingroup cgraph_graph
+ */
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
+#include <assert.h>
 #include <cgraph/cghdr.h>
-#include <stddef.h>
+#include <cgraph/node_set.h>
+#include <limits.h>
+#include <stdbool.h>
+#include <stdlib.h>
+#include <util/alloc.h>
 
 Agraph_t *Ag_G_global;
 
@@ -19,19 +29,12 @@ Agraph_t *Ag_G_global;
  */
 static Agclos_t *agclos(Agdisc_t * proto)
 {
-    Agmemdisc_t *memdisc;
-    void *memclosure;
     Agclos_t *rv;
 
     /* establish an allocation arena */
-    memdisc = ((proto && proto->mem) ? proto->mem : &AgMemDisc);
-    memclosure = memdisc->open(proto);
-    rv = memdisc->alloc(memclosure, sizeof(Agclos_t));
-    rv->disc.mem = memdisc;
-    rv->state.mem = memclosure;
+    rv = gv_calloc(1, sizeof(Agclos_t));
     rv->disc.id = ((proto && proto->id) ? proto->id : &AgIdDisc);
     rv->disc.io = ((proto && proto->io) ? proto->io : &AgIoDisc);
-    rv->callbacks_enabled = TRUE;
     return rv;
 }
 
@@ -45,14 +48,14 @@ Agraph_t *agopen(char *name, Agdesc_t desc, Agdisc_t * arg_disc)
     IDTYPE gid;
 
     clos = agclos(arg_disc);
-    g = clos->disc.mem->alloc(clos->state.mem, sizeof(Agraph_t));
+    g = gv_calloc(1, sizeof(Agraph_t));
     AGTYPE(g) = AGRAPH;
     g->clos = clos;
     g->desc = desc;
-    g->desc.maingraph = TRUE;
+    g->desc.maingraph = true;
     g->root = g;
     g->clos->state.id = g->clos->disc.id->open(g, arg_disc);
-    if (agmapnametoid(g, AGRAPH, name, &gid, TRUE))
+    if (agmapnametoid(g, AGRAPH, name, &gid, true))
 	AGID(g) = gid;
     g = agopen1(g);
     agregister(g, AGRAPH, g);
@@ -67,15 +70,25 @@ Agraph_t *agopen1(Agraph_t * g)
     Agraph_t *par;
 
     g->n_seq = agdtopen(g, &Ag_subnode_seq_disc, Dttree);
-    g->n_id = agdtopen(g, &Ag_subnode_id_disc, Dttree);
+    g->n_id = node_set_new();
     g->e_seq = agdtopen(g, g == agroot(g)? &Ag_mainedge_seq_disc : &Ag_subedge_seq_disc, Dttree);
     g->e_id = agdtopen(g, g == agroot(g)? &Ag_mainedge_id_disc : &Ag_subedge_id_disc, Dttree);
-    g->g_dict = agdtopen(g, &Ag_subgraph_id_disc, Dttree);
+    g->g_seq = agdtopen(g, &Ag_subgraph_seq_disc, Dttree);
+
+    // `agdtopen` has allocated a `Dt_t`, which we now expand to host a
+    // `g_seq_t`. See `g_seq_t` for why we do this odd thing.
+    g->g_seq = gv_realloc(g->g_seq, sizeof(Dt_t), sizeof(g_seq_t));
+
+    g->g_id = agdtopen(g, &Ag_subgraph_id_disc, Dttree);
 
     par = agparent(g);
     if (par) {
-	AGSEQ(g) = agnextseq(par, AGRAPH);
-	dtinsert(par->g_dict, g);
+	uint64_t seq = agnextseq(par, AGRAPH);
+	assert((seq & SEQ_MASK) == seq && "sequence ID overflow");
+	AGSEQ(g) = seq & SEQ_MASK;
+	dtinsert(par->g_seq, g);
+	Agraphs_append(g_seq2(par), g);
+	dtinsert(par->g_id, g);
     }
     if (!par || par->desc.has_attrs)
 	agraphattr_init(g);
@@ -92,13 +105,6 @@ int agclose(Agraph_t * g)
     Agnode_t *n, *next_n;
 
     par = agparent(g);
-    if (par == NULL && AGDISC(g, mem)->close) {
-	/* free entire heap */
-	agmethod_delete(g, g);	/* invoke user callbacks */
-	agfreeid(g, AGRAPH, AGID(g));
-	AGDISC(g, mem)->close(AGCLOS(g, mem));	/* whoosh */
-	return SUCCESS;
-    }
 
     for (subg = agfstsubg(g); subg; subg = next_subg) {
 	next_subg = agnxtsubg(subg);
@@ -113,8 +119,8 @@ int agclose(Agraph_t * g)
     aginternalmapclose(g);
     agmethod_delete(g, g);
 
-    assert(dtsize(g->n_id) == 0);
-    if (agdtclose(g, g->n_id)) return FAILURE;
+    assert(node_set_is_empty(g->n_id));
+    node_set_free(&g->n_id);
     assert(dtsize(g->n_seq) == 0);
     if (agdtclose(g, g->n_seq)) return FAILURE;
 
@@ -123,8 +129,16 @@ int agclose(Agraph_t * g)
     assert(dtsize(g->e_seq) == 0);
     if (agdtclose(g, g->e_seq)) return FAILURE;
 
-    assert(dtsize(g->g_dict) == 0);
-    if (agdtclose(g, g->g_dict)) return FAILURE;
+    // needs to be done before closing `g_seq` because `agdtclose(g, g->g_seq)`
+    // deallocates the memory this is hosted in
+    assert(Agraphs_is_empty(g_seq2(g)));
+    Agraphs_free(g_seq2(g));
+
+    assert(dtsize(g->g_seq) == 0);
+    if (agdtclose(g, g->g_seq)) return FAILURE;
+
+    assert(dtsize(g->g_id) == 0);
+    if (agdtclose(g, g->g_id)) return FAILURE;
 
     if (g->desc.has_attrs)
 	if (agraphattr_delete(g)) return FAILURE;
@@ -135,17 +149,14 @@ int agclose(Agraph_t * g)
 	agdelsubg(par, g);
 	agfree(par, g);
     } else {
-	Agmemdisc_t *memdisc;
-	void *memclos, *clos;
+	void *clos;
 	while (g->clos->cb)
 	    agpopdisc(g, g->clos->cb->f);
 	AGDISC(g, id)->close(AGCLOS(g, id));
 	if (agstrclose(g)) return FAILURE;
-	memdisc = AGDISC(g, mem);
-	memclos = AGCLOS(g, mem);
 	clos = g->clos;
-	(memdisc->free) (memclos, g);
-	(memdisc->free) (memclos, clos);
+	free(g);
+	free(clos);
     }
     return SUCCESS;
 }
@@ -157,7 +168,8 @@ uint64_t agnextseq(Agraph_t * g, int objtype)
 
 int agnnodes(Agraph_t * g)
 {
-    return dtsize(g->n_id);
+  assert(node_set_size(g->n_id) <= INT_MAX);
+  return (int)node_set_size(g->n_id);
 }
 
 int agnedges(Agraph_t * g)
@@ -166,13 +178,13 @@ int agnedges(Agraph_t * g)
     int rv = 0;
 
     for (n = agfstnode(g); n; n = agnxtnode(g, n))
-	rv += agdegree(g, n, FALSE, TRUE);	/* must use OUT to get self-arcs */
+	rv += agdegree(g, n, 0, 1);	/* must use OUT to get self-arcs */
     return rv;
 }
 
 int agnsubg(Agraph_t * g)
 {
-	return dtsize(g->g_dict);
+	return dtsize(g->g_seq);
 }
 
 int agisdirected(Agraph_t * g)
@@ -235,34 +247,68 @@ int agdegree(Agraph_t * g, Agnode_t * n, int want_in, int want_out)
 	return rv;
 }
 
-static int agraphidcmpf(Dict_t * d, void *arg0, void *arg1, Dtdisc_t * disc)
-{
-    (void)d; /* unused */
-    (void)disc; /* unused */
-    ptrdiff_t	v;
-    Agraph_t *sg0, *sg1;
-    sg0 = (Agraph_t *) arg0;
-    sg1 = (Agraph_t *) arg1;
-    v = (AGID(sg0) - AGID(sg1));
-    return ((v==0)?0:(v<0?-1:1));
+static int agraphseqcmpf(void *arg0, void *arg1) {
+  Agraph_t *sg0 = arg0;
+  Agraph_t *sg1 = arg1;
+  if (AGSEQ(sg0) < AGSEQ(sg1)) {
+    return -1;
+  }
+  if (AGSEQ(sg0) > AGSEQ(sg1)) {
+    return 1;
+  }
+  return 0;
 }
 
-Dtdisc_t Ag_subgraph_id_disc = {
-    .link = offsetof(Agraph_t, link), // link offset
-    .comparf = agraphidcmpf,
-    .memoryf = agdictobjmem,
+static int agraphidcmpf(void *arg0, void *arg1) {
+    Agraph_t *sg0 = arg0;
+    Agraph_t *sg1 = arg1;
+    if (AGID(sg0) < AGID(sg1)) {
+	return -1;
+    }
+    if (AGID(sg0) > AGID(sg1)) {
+	return 1;
+    }
+    return 0;
+}
+
+Dtdisc_t Ag_subgraph_seq_disc = {
+  .link = offsetof(Agraph_t, seq_link), // link offset
+  .comparf = agraphseqcmpf,
 };
 
-Agdesc_t Agdirected = { .directed = 1, .maingraph = 1 };
-Agdesc_t Agstrictdirected = { .directed = 1, .strict = 1, .maingraph = 1 };
-Agdesc_t Agundirected = { .maingraph = 1 };
-Agdesc_t Agstrictundirected = { .strict = 1, .maingraph = 1 };
+Dtdisc_t Ag_subgraph_id_disc = {
+    .link = offsetof(Agraph_t, id_link), // link offset
+    .comparf = agraphidcmpf,
+};
 
-Agdisc_t AgDefaultDisc = { &AgMemDisc, &AgIdDisc, &AgIoDisc };
+Agdesc_t Agdirected = {.directed = true, .maingraph = true};
+Agdesc_t Agstrictdirected = {.directed = true, .strict = true, .maingraph = true};
+Agdesc_t Agundirected = {.maingraph = true};
+Agdesc_t Agstrictundirected = {.strict = true, .maingraph = true};
+
+Agdisc_t AgDefaultDisc = { &AgIdDisc, &AgIoDisc };
 
 /**
  * @dir lib/cgraph
  * @brief abstract graph C library, API cgraph.h
  *
  * [man 3 cgraph](https://graphviz.org/pdf/cgraph.3.pdf)
+ *
+ * See @ref cgraph
+ *
+ * @defgroup cgraph Cgraph
+ * @brief abstract graph C library, API: @ref cgraph_api
+ *
+ * Public API of the library: @ref cgraph_api
+ *
+ * Layers:
+ *
+ * * top layer: @ref cgraph_app - uncoupled application specific functions
+ * * middle layer: @ref cgraph_core - highly cohesive core
+ * * bottom layer: @ref cgraph_utils
+ *
+ * @{
+ * @defgroup cgraph_core core
+ * @brief highly cohesive core
+ * @}
  */
