@@ -13,13 +13,13 @@
 
 
 #include <assert.h>
-#include <cgraph/startswith.h>
 #include <common/geomprocs.h>
 #include <common/render.h>
 #include <math.h>
 #include <stdbool.h>
 #include <stddef.h>
 #include <stdint.h>
+#include <util/startswith.h>
 #include <util/streq.h>
 
 #define EPSILON .0001
@@ -229,16 +229,11 @@ void arrow_flags(Agedge_t *e, uint32_t *sflag, uint32_t *eflag) {
 	}
     }
     if (*eflag == ARR_TYPE_NORM) {
-	/* we cannot use the pre-constructed E_arrowhead here because the order in
-	 * which edge attributes appear and are thus parsed into a dictionary mean
-	 * E_arrowhead->id potentially points at a stale attribute value entry
-	 */
 	Agsym_t *arrowhead = agfindedgeattr(agraphof(e), "arrowhead");
 	if (arrowhead != NULL && ((attr = agxget(e, arrowhead)))[0])
 		arrow_match_name(attr, eflag);
     }
     if (*sflag == ARR_TYPE_NORM) {
-	/* similar to above, we cannot use E_arrowtail here */
 	Agsym_t *arrowtail = agfindedgeattr(agraphof(e), "arrowtail");
 	if (arrowtail != NULL && ((attr = agxget(e, arrowtail)))[0])
 		arrow_match_name(attr, sflag);
