@@ -1,22 +1,22 @@
-/* $Id$ $Revision$ */
-/* vim:set shiftwidth=4 ts=8: */
-
+/// @file
+/// @brief @ref fdp_parms
+/// @ingroup common_render
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
- * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #include "config.h"
 
 #define EXTERN
-#include "types.h"
-#include "globals.h"
-#include "fdp.h"
+#include <common/types.h>
+#include <common/globals.h>
+#include <fdpgen/fdp.h>
 
 /* Default layout values, possibly set via command line; -1 indicates unset */
 static fdpParms_t fdpParms = {
