@@ -1,9 +1,11 @@
+/// @file
+/// @ingroup common_render
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -14,19 +16,18 @@
 
 #include "config.h"
 #include <assert.h>
-#include <limits.h>
 #include <math.h>
 #include <stdbool.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <types.h>
-#include <common/memory.h>
-#include <cgraph/alloc.h>
-#include <cgraph/agxbuf.h>
 #include <cgraph/list.h>
-#include <cgraph/prisize_t.h>
+#include <common/render.h>
 #include <common/utils.h>
+#include <util/agxbuf.h>
+#include <util/alloc.h>
+#include <util/prisize_t.h>
 
   /* sample point size; should be dynamic based on dpi or under user control */
 #define BEZIERSUBDIVISION 20
@@ -54,21 +55,6 @@ static void addto (stroke_t* p, double x, double y)
     p->vertices[p->nvertices++] = pt;
 }
 
-static void arcn (stroke_t* p, double x, double y, double r, double a1, double a2)
-{
-    double theta;
-    int i;
-
-    addto (p, x+r*cos(a1), y+r*sin(a1));
-    if (r == 0) return;
-    while (a2 > a1) a2 -= 2*M_PI;
-    theta = a1 - a2; 
-    while (theta > 2*M_PI) theta -= 2*M_PI;
-    theta /= BEZIERSUBDIVISION - 1;
-    for (i = 1; i < BEZIERSUBDIVISION; i++)
-	addto (p, x+r*cos(a1-i*theta), y+r*sin(a1-i*theta));
-}
-
 /*
  * handle zeros
  */
@@ -138,22 +124,22 @@ static double l2dist (pointf p0, pointf p1)
  * turn all curves into lines
  */
 static vararr_t pathtolines(bezier *bez) {
-    int i, j, step;
+    int step;
     double seglen, linelen = 0;
     vararr_t arr = {0};
     pointf p0, p1, V[4];
-    int n = bez->size;
+    const size_t n = bez->size;
     pointf* A = bez->list;
 
     insertArr(&arr, A[0], 0);
     V[3] = A[0];
-    for (i = 0; i + 3 < n; i += 3) {
+    for (size_t i = 0; i + 3 < n; i += 3) {
 	V[0] = V[3];
-	for (j = 1; j <= 3; j++)
+	for (size_t j = 1; j <= 3; j++)
 	    V[j] = A[i + j];
 	p0 = V[0];
 	for (step = 1; step <= BEZIERSUBDIVISION; step++) {
-	    p1 = Bezier(V, 3, (double) step / BEZIERSUBDIVISION, NULL, NULL);
+	    p1 = Bezier(V, (double) step / BEZIERSUBDIVISION, NULL, NULL);
 	    seglen = l2dist(p0, p1);
 	    /* If initwid is large, this may never happen, so turn off. I assume this is to prevent
 	     * too man points or too small a movement. Perhaps a better test can be made, but for now
@@ -172,29 +158,16 @@ static vararr_t pathtolines(bezier *bez) {
     return arr;
 }
 
-static void drawbevel(double x, double y, double lineout, int forward, double dir, double dir2, int linejoin, stroke_t* p)
-{
-    double a, a1, a2;
+static void drawbevel(double x, double lineout, bool forward, double dir,
+                      double dir2, stroke_t *p) {
+    double a2;
 
     if (forward) {
-	a1 = dir;
 	a2 = dir2;
     } else {
-	a1 = dir2;
 	a2 = dir;
     }
-    if (linejoin == 1) {
-	a = a1 - a2;
-	if (a <= D2R(0.1)) a += D2R(360);
-	if (a < D2R(180)) {
-	    a1 = a + a2;
-	    arcn (p,x,y,lineout,a1,a2);
-	} else {
-	    lineto (p, x + lineout*cos(a2), x + lineout*sin(a2));
-	}
-    } else {
-	lineto (p, x + lineout*cos(a2), x + lineout*sin(a2));
-    }
+    lineto (p, x + lineout*cos(a2), x + lineout*sin(a2));
 }
 
 typedef double (*radfunc_t) (double curlen, double totallen, double initwid);
@@ -204,13 +177,8 @@ typedef double (*radfunc_t) (double curlen, double totallen, double initwid);
  * edge, starting with width initwid.
  * The radfunc determines the half-width along the curve. Typically, this will
  * decrease from initwid to 0 as the curlen goes from 0 to totallen.
- * The linejoin and linecap parameters have roughly the same meaning as in postscript.
- *  - linejoin = 0 or 1 
- *  - linecap = 0 or 1 or 2 
  */
-stroke_t taper (bezier* bez, radfunc_t radfunc, double initwid, int linejoin, int linecap)
-{
-    int l, n;
+stroke_t taper(bezier *bez, radfunc_t radfunc, double initwid) {
     double direction=0, direction_2=0;
     vararr_t arr = pathtolines(bez);
     pathpoint cur_point, last_point, next_point;
@@ -226,10 +194,8 @@ stroke_t taper (bezier* bez, radfunc_t radfunc, double initwid, int linejoin, in
 
     /* determine miter and bevel points and directions */
     for (size_t i = 0; i < pathcount; i++) {
-	assert(i <= INT_MAX);
-	assert(pathcount <= INT_MAX);
-	l = mymod((int)i - 1, (int)pathcount);
-	n = mymod((int)i + 1, (int)pathcount);
+	const size_t l = i == 0 ? pathcount - 1 : i - 1;
+	const size_t n = (i + 1) % pathcount;
 
 	cur_point = pathpoints[i];
 	x = cur_point.x;
@@ -256,16 +222,8 @@ stroke_t taper (bezier* bez, radfunc_t radfunc, double initwid, int linejoin, in
 	    lineout = linerad;
 	    if (i == 0) {
 		direction = ndir + D2R(90);
-		if (linecap == 2) {
-		    x -= cos(ndir)*lineout;
-		    y -= sin(ndir)*lineout;
-		}
 	    } else {
 		direction = ldir - D2R(90);
-		if (linecap == 2) {
-		    x -= cos(ldir)*lineout;
-		    y -= sin(ldir)*lineout;
-		}
 	    }
 	    direction_2 = direction;
 	} else {
@@ -282,7 +240,7 @@ stroke_t taper (bezier* bez, radfunc_t radfunc, double initwid, int linejoin, in
 	    }
 		 /* direction to junction point */
 	    direction = ndir+D2R(90)+phi;
-	    if (0 != linejoin || lineout > currentmiterlimit * linerad) {
+	    if (lineout > currentmiterlimit * linerad) {
 		bevel = true;
 		lineout = linerad;
 		direction = mymod(ldir-D2R(90),D2R(360));
@@ -321,16 +279,12 @@ stroke_t taper (bezier* bez, radfunc_t radfunc, double initwid, int linejoin, in
 	    lineto(&p, x+cos(direction)*lineout, y+sin(direction)*lineout);
 	}
 	if (bevel) {
-	    drawbevel(x, y, lineout, TRUE, direction, direction_2, linejoin, &p);
+	    drawbevel(x, lineout, true, direction, direction_2, &p);
 	}
     }
 	 /* end circle as needed */
-    if (linecap == 1) {
-	arcn(&p, x,y,lineout,direction,direction+D2R(180));
-    } else {
-	direction += D2R(180);
-	lineto(&p, x+cos(direction)*lineout, y+sin(direction)*lineout);
-    }
+    direction += D2R(180);
+    lineto(&p, x+cos(direction)*lineout, y+sin(direction)*lineout);
 	 /* side 2 */
     assert(pathcount > 0);
     for (size_t i = pathcount - 2; i != SIZE_MAX; i--) {
@@ -343,14 +297,9 @@ stroke_t taper (bezier* bez, radfunc_t radfunc, double initwid, int linejoin, in
 	direction_2 = cur_point.dir2 + D2R(180);
 	lineto(&p, x+cos(direction_2)*lineout, y+sin(direction_2)*lineout);
 	if (bevel) { 
-	    drawbevel(x, y, lineout, FALSE, direction, direction_2, linejoin, &p);
+	    drawbevel(x, lineout, false, direction, direction_2, &p);
 	}
     }
-	 /* start circle if needed */
-    if (linecap == 1) {
-	arcn(&p, x,y,lineout,direction,direction+D2R(180));
-    }
-    /* closepath(&p); */
     free(pathpoints);
     return p;
 }
@@ -375,7 +324,7 @@ main ()
 
     bez.size = sizeof(pts)/sizeof(pointf);
     bez.list = pts;
-    sp = taper(&bez, halffunc, 20.0, 0, 0);
+    sp = taper(&bez, halffunc, 20.0);
     printf ("newpath\n");
     printf ("%.02f %.02f moveto\n", sp->vertices[0].x, sp->vertices[0].y);
     for (size_t i = 1; i < sp->nvertices; i++)
