@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -23,18 +23,21 @@
  *  watch out for interactions between leaves and clusters.
  */
 
+#include	<cgraph/gv_math.h>
 #include	<dotgen/dot.h>
 #include	<limits.h>
 #include	<stdbool.h>
+#include	<stddef.h>
+#include	<stdint.h>
+#include	<util/alloc.h>
 
-static void dot1_rank(graph_t * g, aspect_t* asp);
-static void dot2_rank(graph_t * g, aspect_t* asp);
+static void dot1_rank(graph_t *g);
+static void dot2_rank(graph_t *g);
 
 static void 
 renewlist(elist * L)
 {
-    int i;
-    for (i = L->size; i >= 0; i--)
+    for (size_t i = L->size; i != SIZE_MAX; i--)
 	L->list[i] = NULL;
     L->size = 0;
 }
@@ -44,14 +47,13 @@ cleanup1(graph_t * g)
 {
     node_t *n;
     edge_t *e, *f;
-    int c;
 
-    for (c = 0; c < GD_comp(g).size; c++) {
+    for (size_t c = 0; c < GD_comp(g).size; c++) {
 	    GD_nlist(g) = GD_comp(g).list[c];
 	    for (n = GD_nlist(g); n; n = ND_next(n)) {
 	        renewlist(&ND_in(n));
 	        renewlist(&ND_out(n));
-	        ND_mark(n) = FALSE;
+	        ND_mark(n) = false;
 	    }
     }
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
@@ -159,7 +161,8 @@ make_new_cluster(graph_t * g, graph_t * subg)
 {
     int cno;
     cno = ++(GD_n_cluster(g));
-    GD_clust(g) = ZALLOC(cno + 1, GD_clust(g), graph_t *, GD_n_cluster(g));
+    GD_clust(g) = gv_recalloc(GD_clust(g), GD_n_cluster(g), cno + 1,
+                              sizeof(graph_t*));
     GD_clust(g)[cno] = subg;
     do_graph_label(subg);
     return cno;
@@ -259,7 +262,7 @@ collapse_cluster(graph_t * g, graph_t * subg)
 	return;
     make_new_cluster(g, subg);
     if (CL_type == LOCAL) {
-	dot1_rank(subg, 0);
+	dot1_rank(subg);
 	cluster_leader(subg);
     } else
 	dot_scan_ranks(subg);
@@ -370,12 +373,11 @@ minmax_edges2(graph_t * g, point slen)
 void rank1(graph_t * g)
 {
     int maxiter = INT_MAX;
-    int c;
     char *s;
 
     if ((s = agget(g, "nslimit1")))
-	maxiter = atof(s) * agnnodes(g);
-    for (c = 0; c < GD_comp(g).size; c++) {
+	maxiter = scale_clamp(agnnodes(g), atof(s));
+    for (size_t c = 0; c < GD_comp(g).size; c++) {
 	GD_nlist(g) = GD_comp(g).list[c];
 	rank(g, (GD_n_cluster(g) == 0 ? 1 : 0), maxiter);	/* TB balance */
     }
@@ -387,8 +389,7 @@ void rank1(graph_t * g)
  * Leaf sets and clusters remain merged.
  * Sets minrank and maxrank appropriately.
  */
-static void expand_ranksets(graph_t * g, aspect_t* asp)
-{
+static void expand_ranksets(graph_t *g) {
     int c;
     node_t *n, *leader;
 
@@ -400,7 +401,7 @@ static void expand_ranksets(graph_t * g, aspect_t* asp)
 	    /* The following works because ND_rank(n) == 0 if n is not in a
 	     * cluster, and ND_rank(n) = the local rank offset if n is in
 	     * a cluster. */
-	    if ((leader != n) && (!asp || (ND_rank(n) == 0)))
+	    if (leader != n)
 		ND_rank(n) += ND_rank(leader);
 
 	    if (GD_maxrank(g) < ND_rank(n))
@@ -425,76 +426,33 @@ static void expand_ranksets(graph_t * g, aspect_t* asp)
     }
 }
 
-#ifdef ALLOW_LEVELS
-void
-setRanks (graph_t* g, attrsym_t* lsym)
-{
-    node_t* n;
-    char*   s;
-    char*   ep;
-    long    v;
-
-    for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
-	s = agxget (n, lsym);
-	v = strtol (s, &ep, 10);
-	if (ep == s)
-	    agerr(AGWARN, "no level attribute for node \"%s\"\n", agnameof(n));
-	ND_rank(n) = v;
-    }
-}
-#endif
-
-/* dot1_rank:
- * asp != NULL => g is root
- */
-static void dot1_rank(graph_t * g, aspect_t* asp)
+static void dot1_rank(graph_t *g)
 {
     point p;
-#ifdef ALLOW_LEVELS
-    attrsym_t* N_level;
-#endif
     edgelabel_ranks(g);
 
-    if (asp) {
-	init_UF_size(g);
-	initEdgeTypes(g);
-    }
-
     collapse_sets(g,g);
     /*collapse_leaves(g); */
     class1(g);
     p = minmax_edges(g);
     decompose(g, 0);
-    if (asp && ((GD_comp(g).size > 1)||(GD_n_cluster(g) > 0))) {
-	asp->badGraph = 1;
-	asp = NULL;
-    }
     acyclic(g);
     if (minmax_edges2(g, p))
 	decompose(g, 0);
-#ifdef ALLOW_LEVELS
-    if ((N_level = agattr(g,AGNODE,"level",NULL)))
-	setRanks(g, N_level);
-    else
-#endif
 
-    if (asp)
-	rank3(g, asp);
-    else
-	rank1(g);
+    rank1(g);
 
-    expand_ranksets(g, asp);
+    expand_ranksets(g);
     cleanup1(g);
 }
 
-void dot_rank(graph_t * g, aspect_t* asp)
-{
-    if (agget (g, "newrank")) {
+void dot_rank(graph_t *g) {
+    if (mapbool(agget(g, "newrank"))) {
 	GD_flags(g) |= NEW_RANK;
-	dot2_rank (g, asp);
+	dot2_rank(g);
     }
     else
-	dot1_rank (g, asp);
+	dot1_rank(g);
     if (Verbose)
 	fprintf (stderr, "Maxrank = %d, minrank = %d\n", GD_maxrank(g), GD_minrank(g));
 }
@@ -536,7 +494,7 @@ static bool is_empty(graph_t *g) {
 static bool is_a_strong_cluster(graph_t * g)
 {
     char *str = agget(g, "compact");
-    return mapBool(str, false);
+    return mapbool(str);
 }
 
 static int rankset_kind(graph_t * g)
@@ -638,16 +596,14 @@ static void compile_samerank(graph_t * ug, graph_t * parent_clust)
 
     /* process this subgraph as a rankset */
     switch (rankset_kind(ug)) {
-    case SOURCERANK:
-	GD_has_sourcerank(clust) = true;	/* fall through */
+    case SOURCERANK: // fall through
     case MINRANK:
 	leader = union_all(ug);
 	if (clust != NULL) {
 	    GD_minrep(clust) = union_one(leader, GD_minrep(clust));
 	}
 	break;
-    case SINKRANK:
-	GD_has_sinkrank(clust) = true;	/* fall through */
+    case SINKRANK: // fall through
     case MAXRANK:
 	leader = union_all(ug);
 	if (clust != NULL) {
@@ -661,7 +617,7 @@ static void compile_samerank(graph_t * ug, graph_t * parent_clust)
     case NORANK:
 	break;
     default:			/* unrecognized - warn and do nothing */
-	agerr(AGWARN, "%s has unrecognized rank=%s", agnameof(ug),
+	agwarningf("%s has unrecognized rank=%s", agnameof(ug),
 	      agget(ug, "rank"));
     }
 
@@ -747,7 +703,7 @@ static void strong(graph_t * g, node_t * t, node_t * h, edge_t * orig)
 	(e = agfindedge(g, h, t)) || (e = agedge(g, t, h, 0, 1)))
 	merge(e, ED_minlen(orig), ED_weight(orig));
     else
-	agerr(AGERR, "ranking: failure to create strong constraint edge between nodes %s and %s\n", 
+	agerrorf("ranking: failure to create strong constraint edge between nodes %s and %s\n", 
 	    agnameof(t), agnameof(h));
 }
 
@@ -865,7 +821,7 @@ static void dfs(graph_t * g, node_t * v)
 
     if (ND_mark(v))
 	return;
-    ND_mark(v) = TRUE;
+    ND_mark(v) = true;
     ND_onstack(v) = true;
     for (e = agfstout(g, v); e; e = f) {
 	f = agnxtout(g, e);
@@ -885,7 +841,7 @@ static void break_cycles(graph_t * g)
     node_t *n;
 
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
-	ND_mark(n) = FALSE;
+	ND_mark(n) = false;
 	ND_onstack(n) = false;
     }
     for (n = agfstnode(g); n; n = agnxtnode(g, n))
@@ -941,7 +897,7 @@ static void readout_levels(graph_t * g, graph_t * Xg, int ncc)
     GD_maxrank(g) = -1;
     if (ncc > 1) {
 	int i;
-	minrk = N_NEW(ncc+1,int);
+	minrk = gv_calloc(ncc + 1, sizeof(int));
 	for (i = 1; i <= ncc; i++)
 	    minrk[i] = INT_MAX;
     }
@@ -983,8 +939,7 @@ static void readout_levels(graph_t * g, graph_t * Xg, int ncc)
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	ND_alg(n) = NULL;
     }
-    if (minrk)
-	free (minrk);
+    free(minrk);
 }
 
 static void dfscc(graph_t * g, node_t * n, int cc)
@@ -1048,8 +1003,7 @@ int infosizes[] = {
     sizeof(Agedgeinfo_t)
 };
 
-void dot2_rank(graph_t * g, aspect_t* asp)
-{
+void dot2_rank(graph_t *g) {
     int ssize;
     int ncc, maxiter = INT_MAX;
     char *s;
@@ -1063,7 +1017,7 @@ void dot2_rank(graph_t * g, aspect_t* asp)
     edgelabel_ranks(g);
 
     if ((s = agget(g, "nslimit1")))
-	maxiter = atof(s) * agnnodes(g);
+	maxiter = scale_clamp(agnnodes(g), atof(s));
     else
 	maxiter = INT_MAX;
 
@@ -1075,11 +1029,6 @@ void dot2_rank(graph_t * g, aspect_t* asp)
     ncc = connect_components(Xg);
     add_fast_edges (Xg);
 
-    if (asp) {
-	init_UF_size(Xg);
-	initEdgeTypes(Xg);
-    }
-
     if ((s = agget(g, "searchsize")))
 	ssize = atoi(s);
     else
