@@ -8,6 +8,8 @@
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
+#pragma once
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -19,22 +21,18 @@ extern "C" {
  * expression library definitions
  */
 
-#ifndef _EXPR_H
-#define _EXPR_H
-
 #include <ast/ast.h>
 #include <inttypes.h>
 
 #include <expr/exparse.h>
 
-#undef	extern
-
 #include <assert.h>
 #include <cdt.h>
-#include <cgraph/agxbuf.h>
 #include <stdarg.h>
+#include <stdbool.h>
 #include <stddef.h>
 #include <stdio.h>
+#include <util/agxbuf.h>
 #include <vmalloc/vmalloc.h>
 
 #define EX_VERSION	20000101L
@@ -52,7 +50,7 @@ extern "C" {
 
 #define EX_NAMELEN	32		/* default Exid_t.name length	*/
 
-#define EX_ID(n,l,i,t)	{{0},(l),(i),(t),0,0,0,0,0,n}
+#define EX_ID(n, l, i, t) {{0}, (l), (i), (t), 0, 0, 0, 0, n}
 
 #define DELETE_T		MINTOKEN		/* exexpr() delete `type'	*/
 
@@ -89,7 +87,7 @@ typedef struct Expr_s Expr_t;
 typedef struct Exref_s Exref_t;
 
 typedef void (*Exerror_f) (Expr_t *, Exdisc_t *, int, const char *, ...);
-typedef void (*Exexit_f) (Expr_t *, Exdisc_t *, int);
+typedef void (*Exexit_f)(void *, int);
 
 typedef struct Exid_s			/* id symbol table info		*/
 {
@@ -101,7 +99,6 @@ typedef struct Exid_s			/* id symbol table info		*/
 	long		flags;		/* user defined flags		*/
 	Exnode_t*	value;		/* value			*/
 	void *local; ///< user defined local stuff
-	long		isstatic;	/* static			*/
 	char		name[EX_NAMELEN];/* symbol name			*/
 } Exid_t;
 
@@ -152,9 +149,8 @@ union Exdata_u
 struct Exnode_s				/* expression tree node		*/
 {
 	long	type; ///< value type
-	int	op;		/* operator			*/
-	int	binary;		/* data.operand.{left,right} ok	*/
-	void *local; ///< user defined local stuff
+	long op; ///< operator
+	bool binary; ///< data.operand.{left,right} ok
 	union
 	{
 	double	(*floating)(char**);	/* FLOATING return value	*/
@@ -175,8 +171,6 @@ struct Exdisc_s				/* discipline			*/
 	uint64_t	flags;		/* EX_* flags			*/
 	Exid_t*		symbols;	/* static symbols		*/
 	char**		data;		/* compiled function arg data	*/
-	char*		lib;		/* pathfind() lib		*/
-	char*		type;		/* pathfind() type		*/
 	int		(*castf)(Expr_t*, Exnode_t*, const char*, int, Exid_t*, int, Exdisc_t*);
 					/* unknown cast function	*/
 	int (*convertf)(Exnode_t *, long, int);
@@ -247,7 +241,7 @@ extern void		exwarn(const char *, ...);
 extern Extype_t		exeval(Expr_t*, Exnode_t*, void*);
 extern Exnode_t*	exexpr(Expr_t*, const char*, Exid_t*, int);
 extern void		exfreenode(Expr_t*, Exnode_t*);
-extern Exnode_t*	exnewnode(Expr_t*, int, int, long, Exnode_t*, Exnode_t*);
+extern Exnode_t *exnewnode(Expr_t *, long, bool, long, Exnode_t *, Exnode_t *);
 extern char*		exnospace(void);
 extern Expr_t*		exopen(Exdisc_t*);
 extern int		expop(Expr_t*);
@@ -257,13 +251,11 @@ extern char*		exstring(Expr_t *, char *);
 extern void*		exstralloc(Expr_t *, size_t);
 extern char* extype(long int);
 extern Extype_t exzero(long int);
-extern char*	exopname(int);
+extern char *exopname(long);
 extern void		exinit(void);
 extern char *extypename(Expr_t *p, long);
 extern int		exisAssign(Exnode_t *);
 
-#undef	extern
-
 /** Construct a vmalloc-backed string.
  *
  * \param vm Allocator to use.
@@ -300,8 +292,6 @@ static inline char *exprintf(Vmalloc_t *vm, const char *fmt, ...) {
   return s;
 }
 
-#endif
-
 #ifdef __cplusplus
 }
 #endif
