@@ -1,9 +1,14 @@
+/**
+ * @file
+ * @brief API fdpgen/fdp.h: @ref fdp_init_node_edge, @ref fdp_cleanup
+ */
+
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -18,11 +23,11 @@
 /* uses PRIVATE interface */
 #define FDP_PRIVATE 1
 
-#include    <cgraph/alloc.h>
 #include    <fdpgen/tlayout.h>
 #include    <neatogen/neatoprocs.h>
-#include    <cgraph/agxbuf.h>
 #include    <stdbool.h>
+#include    <util/agxbuf.h>
+#include    <util/alloc.h>
 
 static void initialPositions(graph_t * g)
 {
@@ -87,7 +92,7 @@ void fdp_init_node_edge(graph_t * g)
     int nn;
     int i;
 
-    aginit(g, AGNODE, "Agnodeinfo_t", sizeof(Agnodeinfo_t), TRUE);
+    aginit(g, AGNODE, "Agnodeinfo_t", sizeof(Agnodeinfo_t), true);
     processClusterEdges(g);
 
     /* Get node count after processClusterEdges(), as this function may
@@ -148,3 +153,13 @@ void fdp_cleanup(graph_t * g)
     }
     fdp_cleanup_graph(g);
 }
+
+/**
+ * @dir lib/fdpgen
+ * @brief [Force-Directed Placement](https://en.wikipedia.org/wiki/Force-directed_graph_drawing) layout engine, API fdpgen/fdp.h
+ * @ingroup engines
+ *
+ * [FDP layout user manual](https://graphviz.org/docs/layouts/fdp/)
+ *
+ * Other @ref engines
+ */
