@@ -16,8 +16,6 @@
 #include <assert.h>
 #include <cgraph/gv_ctype.h>
 #include <neatogen/neato.h>
-#include <cgraph/agxbuf.h>
-#include <cgraph/startswith.h>
 #include <common/utils.h>
 #include <float.h>
 #include <math.h>
@@ -36,7 +34,9 @@
 #include <neatogen/quad_prog_vpsc.h>
 #endif
 #include <stddef.h>
+#include <util/agxbuf.h>
 #include <util/alloc.h>
+#include <util/startswith.h>
 #include <util/strcasecmp.h>
 
 #define SEPFACT         0.8 // default esep/sep
