@@ -3,13 +3,15 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #pragma once
 
+#include <stddef.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -22,16 +24,14 @@ extern "C" {
     } Point;
 #endif
 
-    extern Point origin;
-
     extern double xmin, xmax, ymin, ymax;	/* extreme x,y values of sites */
-    extern double deltax, deltay;	/* xmax - xmin, ymax - ymin */
+    extern double deltax;	// xmax - xmin
 
-    extern int nsites;		/* Number of sites */
+    extern size_t nsites; // Number of sites
     extern int sqrt_nsites;
 
     extern void geominit(void);
-    extern double dist_2(Point *, Point *);	/* Distance squared between two points */
+    extern double dist_2(Point, Point); ///< distance squared between two points
     extern void subpt(Point * a, Point b, Point c);
     extern void addpt(Point * a, Point b, Point c);
     extern double area_2(Point a, Point b, Point c);
