@@ -3,13 +3,15 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #pragma once
 
+#include <stddef.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -19,8 +21,8 @@ extern "C" {
     /* Sites are also used as vertices on line segments */
     typedef struct Site {
 	Point coord;
-	int sitenbr;
-	int refcnt;
+	size_t sitenbr;
+	unsigned refcnt;
     } Site;
 
     extern int siteidx;
