@@ -3,15 +3,12 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #include "config.h"
-#include <cgraph/agxbuf.h>
-#include <cgraph/alloc.h>
-#include <cgraph/bitarray.h>
 #include <cgraph/list.h>
 #include <sparse/SparseMatrix.h>
 #include <sfdpgen/spring_electrical.h>
@@ -27,28 +24,39 @@
 #include <stddef.h>
 #include <string.h>
 #include <time.h>
+#include <util/agxbuf.h>
+#include <util/alloc.h>
+#include <util/bitarray.h>
 
-#define PI M_PI
+/// another parameter
+/// fₐ(i, j) = C × dist(i , j)² ÷ K × dᵢⱼ, fᵣ(i, j) = K³⁻ᵖ ÷ dist(i, j)⁻ᵖ
+static const double C = 0.2;
+
+/// cut off size above which quadtree approximation is used
+static const int quadtree_size = 45;
+
+/// Barnes-Hutt constant,
+/// if width(snode) ÷ dist[i, snode] < bh, treat snode as a supernode.
+static const double bh = 0.6;
+
+/// minimum different between two subsequence config before terminating.
+/// ||x - xold||_∞ < tol ÷ K
+static const double tol = 0.001;
+
+static const double cool = 0.90;
 
 spring_electrical_control spring_electrical_control_new(void){
   spring_electrical_control ctrl;
   ctrl = gv_alloc(sizeof(struct spring_electrical_control_struct));
   ctrl->p = AUTOP;/*a negativve number default to -1. repulsive force = dist^p */
-  ctrl->q = 1;/*a positive number default to 1. Only apply to maxent.
-		attractive force = dist^q. Stress energy = (||x_i-x_j||-d_ij)^{q+1} */
-  ctrl->random_start = TRUE;/* whether to apply SE from a random layout, or from exisiting layout */
+  ctrl->random_start = true; // whether to apply SE from a random layout, or from existing layout
   ctrl->K = -1;/* the natural distance. If K < 0, K will be set to the average distance of an edge */
-  ctrl->C = 0.2;/* another parameter. f_a(i,j) = C*dist(i,j)^2/K * d_ij, f_r(i,j) = K^(3-p)/dist(i,j)^(-p). By default C = 0.2. */
-  ctrl->multilevels = FALSE;/* if <=1, single level */
+  ctrl->multilevels = 0;/* if <=1, single level */
 
-  ctrl->quadtree_size = 45;/* cut off size above which quadtree approximation is used */
   ctrl->max_qtree_level = 10;/* max level of quadtree */
-  ctrl->bh = 0.6;/* Barnes-Hutt constant, if width(snode)/dist[i,snode] < bh, treat snode as a supernode.*/
-  ctrl->tol = 0.001;/* minimum different between two subsequence config before terminating. ||x-xold||_infinity < tol/K */
   ctrl->maxiter = 500;
-  ctrl->cool = 0.90;/* default 0.9 */
   ctrl->step = 0.1;
-  ctrl->adaptive_cooling = TRUE;
+  ctrl->adaptive_cooling = true;
   ctrl->random_seed = 123;
   ctrl->beautify_leaves = false;
   ctrl->smoothing = SMOOTHING_NONE;
@@ -75,13 +83,15 @@ static char* tschemes[] = {
 
 void spring_electrical_control_print(spring_electrical_control ctrl){
   fprintf (stderr, "spring_electrical_control:\n");
-  fprintf (stderr, "  repulsive and attractive exponents: %.03f %.03f\n", ctrl->p, ctrl->q);
-  fprintf (stderr, "  random start %d seed %d\n", ctrl->random_start, ctrl->random_seed);
-  fprintf (stderr, "  K : %.03f C : %.03f\n", ctrl->K, ctrl->C);
+  fprintf (stderr, "  repulsive exponent: %.03f\n", ctrl->p);
+  fprintf(stderr, "  random start %d seed %d\n", (int)ctrl->random_start,
+          ctrl->random_seed);
+  fprintf (stderr, "  K : %.03f C : %.03f\n", ctrl->K, C);
   fprintf (stderr, "  max levels %d\n", ctrl->multilevels);
-  fprintf (stderr, "  quadtree size %d max_level %d\n", ctrl->quadtree_size, ctrl->max_qtree_level);
-  fprintf (stderr, "  Barnes-Hutt constant %.03f tolerance  %.03f maxiter %d\n", ctrl->bh, ctrl->tol, ctrl->maxiter);
-  fprintf (stderr, "  cooling %.03f step size  %.03f adaptive %d\n", ctrl->cool, ctrl->step, ctrl->adaptive_cooling);
+  fprintf (stderr, "  quadtree size %d max_level %d\n", quadtree_size, ctrl->max_qtree_level);
+  fprintf (stderr, "  Barnes-Hutt constant %.03f tolerance  %.03f maxiter %d\n", bh, tol, ctrl->maxiter);
+  fprintf(stderr, "  cooling %.03f step size  %.03f adaptive %d\n", cool,
+          ctrl->step, (int)ctrl->adaptive_cooling);
   fprintf (stderr, "  beautify_leaves %d node weights %d rotation %.03f\n",
            (int)ctrl->beautify_leaves, 0, ctrl->rotation);
   fprintf (stderr, "  smoothing %s overlap %d initial_scaling %.03f do_shrinking %d\n",
@@ -90,19 +100,22 @@ void spring_electrical_control_print(spring_electrical_control ctrl){
   fprintf (stderr, "  edge_labeling_scheme %d\n", ctrl->edge_labeling_scheme);
 }
 
-void oned_optimizer_delete(oned_optimizer opt){
-  free(opt);
-}
+enum { MAX_I = 20, OPT_UP = 1, OPT_DOWN = -1, OPT_INIT = 0 };
 
-oned_optimizer oned_optimizer_new(int i){
-  oned_optimizer opt;
-  opt = gv_alloc(sizeof(struct oned_optimizer_struct));
-  opt->i = i;
-  opt->direction = OPT_INIT;
+typedef struct {
+  int i;
+  double work[MAX_I + 1];
+  int direction;
+} oned_optimizer;
+
+static oned_optimizer oned_optimizer_new(int i){
+  oned_optimizer opt = {0};
+  opt.i = i;
+  opt.direction = OPT_INIT;
   return opt;
 }
 
-void oned_optimizer_train(oned_optimizer opt, double work){
+static void oned_optimizer_train(oned_optimizer *opt, double work) {
   int i = opt->i;
 
   assert(i >= 0);
@@ -116,32 +129,26 @@ void oned_optimizer_train(oned_optimizer opt, double work){
       opt->i = MIN(MAX_I, opt->i + 1);
     }
   } else if (opt->direction == OPT_UP){
-    /*    fprintf(stderr, "{current_level, prev_level} = {%d,%d}, {work, work_prev} = {%f,%f}",i,i-1,opt->work[i], opt->work[i-1]);*/
     assert(i >= 1);
     if (opt->work[i] < opt->work[i-1] && opt->i < MAX_I){
-      /*      fprintf(stderr, "keep going up to level %d\n",opt->i+1);*/
       opt->i = MIN(MAX_I, opt->i + 1);
     } else {
-      /*      fprintf(stderr, "going down to level %d\n",opt->i-1);*/
       (opt->i)--;
       opt->direction = OPT_DOWN;
     }
   } else {
     assert(i < MAX_I);
-    /*    fprintf(stderr, "{current_level, prev_level} = {%d,%d}, {work, work_prev} = {%f,%f}",i,i+1,opt->work[i], opt->work[i+1]);*/
     if (opt->work[i] < opt->work[i+1] && opt->i > 0){
-      /*      fprintf(stderr, "keep going down to level %d\n",opt->i-1);*/
       opt->i = MAX(0, opt->i-1);
     } else {
-      /*      fprintf(stderr, "keep up to level %d\n",opt->i+1);*/
       (opt->i)++;
       opt->direction = OPT_UP;
     }
   }
 }
 
-int oned_optimizer_get(oned_optimizer opt){
-  return opt->i;
+static int oned_optimizer_get(const oned_optimizer opt) {
+  return opt.i;
 }
 
 
@@ -163,128 +170,8 @@ double average_edge_length(SparseMatrix A, int dim, double *coord){
   return dist/ia[A->m];
 }
 
-#ifdef ENERGY
-static double spring_electrical_energy(int dim, SparseMatrix A, double *x, double p, double CRK, double KP){
-      /* 1. Grad[||x-y||^k,x] = k||x-y||^(k-1)*0.5*(x-y)/||x-y|| = k/2*||x-y||^(k-2) (x-y)
-	 which should equal to -force (force = -gradient),
-	 hence energy for force ||x-y||^m (y-x) is ||x-y||^(m+2)*2/(m+2) where m != 2
-	 2. Grad[Log[||x-y||],x] = 1/||x-y||*0.5*(x-y)/||x-y|| = 0.5*(x-y)/||x-y||^2,
-	 hence the energy to give force ||x-y||^-2 (x-y) is -2*Log[||x-y||]
-
-      */
-  int i, j, k, *ia = A->ia, *ja = A->ja, n = A->m;
-  double energy = 0, dist;
-
-  for (i = 0; i < n; i++){
-    /* attractive force   C^((2-p)/3) ||x_i-x_j||/K * (x_j - x_i) */
-    for (j = ia[i]; j < ia[i+1]; j++){
-      if (ja[j] == i) continue;
-      dist = distance(x, dim, i, ja[j]);
-      energy += CRK*pow(dist, 3.)*2./3.;
-    }
-
-    /* repulsive force K^(1 - p)/||x_i-x_j||^(1 - p) (x_i - x_j) */
-    for (j = 0; j < n; j++){
-      if (j == i) continue;
-      dist = distance_cropped(x, dim, i, j);
-      for (k = 0; k < dim; k++){
-	if (p == -1){
-	  energy  += -KP*2*log(dist);
-	} else {
-	  energy += -KP*pow(dist,p+1)*2/(p+1);
-	}
-      }
-    }
-  }
-  return energy;
-}
-
-#endif
-
-void export_embedding(FILE *fp, int dim, SparseMatrix A, double *x, double *width){
-  int i, j, k, *ia=A->ia, *ja = A->ja;
-  int ne = 0;
-  double xsize, ysize, xmin, xmax, ymin, ymax;
-
-  xmax = xmin = x[0];
-  ymax = ymin = x[1];
-  for (i = 0; i < A->m; i++){
-    xmax = fmax(xmax, x[i*dim]);
-    xmin = fmin(xmin, x[i*dim]);
-    ymax = fmax(ymax, x[i*dim+1]);
-    ymin = fmin(ymin, x[i*dim+1]);
-  }
-  xsize = xmax-xmin;
-  ysize = ymax-ymin;
-  xsize = fmax(xsize, ysize);
-
-  if (dim == 2){
-    fprintf(fp,"Graphics[{GrayLevel[0.5],Line[{");
-  } else {
-    fprintf(fp,"Graphics3D[{GrayLevel[0.5],Line[{");
-  }
-  for (i = 0; i < A->m; i++){
-    for (j = ia[i]; j < ia[i+1]; j++){
-      if (ja[j] == i) continue;
-      ne++;
-      if (ne > 1) fprintf(fp, ",");
-      fprintf(fp, "{{");
-      for (k = 0; k < dim; k++) {
-	if (k > 0) fprintf(fp,",");
-	fprintf(fp, "%f",x[i*dim+k]);
-      }
-      fprintf(fp, "},{");
-      for (k = 0; k < dim; k++) {
-	if (k > 0) fprintf(fp,",");
-	fprintf(fp, "%f",x[ja[j]*dim+k]);
-      }
-      fprintf(fp, "}}");
-    }
-  }
-
-  fprintf(fp,"}],Hue[%f]",/*drand()*/1.);
-
-  if (width && dim == 2){
-    for (i = 0; i < A->m; i++){
-      if (i >= 0) fprintf(fp,",");
-      fprintf(fp,"(*width={%f,%f}, x = {%f,%f}*){GrayLevel[.5,.5],Rectangle[{%f,%f},{%f,%f}]}", width[i*dim], width[i*dim+1], x[i*dim], x[i*dim + 1],
-          x[i*dim] - width[i*dim], x[i*dim+1] - width[i*dim+1],
-          x[i*dim] + width[i*dim], x[i*dim+1] + width[i*dim+1]);
-    }
-  }
-
-  if (A->m < 100){
-    for (i = 0; i < A->m; i++){
-      if (i >= 0) fprintf(fp,",");
-      fprintf(fp,"Text[%d,{",i+1);
-      for (k = 0; k < dim; k++) {
-	if (k > 0) fprintf(fp,",");
-	fprintf(fp, "%f",x[i*dim+k]);
-      }
-      fprintf(fp,"}]");
-    }
-  } else if (A->m < 500000){
-    fprintf(fp, ", Point[{");
-    for (i = 0; i < A->m; i++){
-      if (i > 0) fprintf(fp,",");
-      fprintf(fp,"{");
-      for (k = 0; k < dim; k++) {
-	if (k > 0) fprintf(fp,",");
-	fprintf(fp, "%f",x[i*dim+k]);
-      }
-      fprintf(fp,"}");
-    }
-    fprintf(fp, "}]");
-  } else {
-      fprintf(fp,"{}");
-  }
-
-
- fprintf(fp,"},ImageSize->%f]\n", 2*xsize/2);
-
-}
-
-static double update_step(int adaptive_cooling, double step, double Fnorm, double Fnorm0, double cool){
+static double update_step(bool adaptive_cooling, double step, double Fnorm,
+                          double Fnorm0) {
 
   if (!adaptive_cooling) {
     return cool*step;
@@ -356,70 +243,31 @@ static void beautify_leaves(int dim, SparseMatrix A, double *x){
   bitarray_reset(&checked);
 }
 
-void force_print(FILE *fp, int n, int dim, double *x, double *force){
-  int i, k;
-
-  fprintf(fp,"Graphics[{");
-  for (i = 0; i < n; i++){
-    if (i > 0) fprintf(fp, ",");
-    fprintf(fp, "Arrow[{{");
-    for (k = 0; k < dim; k++){
-      if (k > 0) fprintf(fp, ",");
-      fprintf(fp, "%f",x[i*dim+k]);
-    }
-    fprintf(fp, "},{");
-    for (k = 0; k < dim; k++){
-      if (k > 0) fprintf(fp, ",");
-      fprintf(fp, "%f",x[i*dim+k]+0.5*force[i*dim+k]);
-    }
-    fprintf(fp, "}}]");
-  }
-  fprintf(fp,",");
-  for (i = 0; i < n; i++){
-    if (i > 0) fprintf(fp, ",");
-    fprintf(fp, "Tooltip[Point[{");
-    for (k = 0; k < dim; k++){
-      if (k > 0) fprintf(fp, ",");
-      fprintf(fp, "%f",x[i*dim+k]);
-    }
-    fprintf(fp, "}],%d]",i);
-  }
-
-
-
-
-  fprintf(fp,"}]\n");
-
-}
-
-
 void spring_electrical_embedding_fast(int dim, SparseMatrix A0, spring_electrical_control ctrl, double *x, int *flag){
   /* x is a point to a 1D array, x[i*dim+j] gives the coordinate of the i-th node at dimension j.  */
   SparseMatrix A = A0;
   int m, n;
   int i, j, k;
-  double p = ctrl->p, K = ctrl->K, C = ctrl->C, CRK, tol = ctrl->tol, maxiter = ctrl->maxiter, cool = ctrl->cool, step = ctrl->step, KP;
+  double p = ctrl->p, K = ctrl->K, CRK, maxiter = ctrl->maxiter, step = ctrl->step, KP;
   int *ia = NULL, *ja = NULL;
   double *f = NULL, dist, F, Fnorm = 0, Fnorm0;
   int iter = 0;
-  int adaptive_cooling = ctrl->adaptive_cooling;
-  QuadTree qt = NULL;
+  const bool adaptive_cooling = ctrl->adaptive_cooling;
   double counts[4], *force = NULL;
 #ifdef TIME
   clock_t start, end, start0;
-  double qtree_cpu = 0, qtree_cpu0 = 0, qtree_new_cpu = 0, qtree_new_cpu0 = 0;
+  double qtree_cpu = 0, qtree_new_cpu = 0;
   double total_cpu = 0;
   start0 = clock();
 #endif
   int max_qtree_level = ctrl->max_qtree_level;
-  oned_optimizer qtree_level_optimizer = NULL;
 
   if (!A || maxiter <= 0) return;
 
   m = A->m, n = A->n;
   if (n <= 0 || dim <= 0) return;
 
-  qtree_level_optimizer = oned_optimizer_new(max_qtree_level);
+  oned_optimizer qtree_level_optimizer = oned_optimizer_new(max_qtree_level);
 
   *flag = 0;
   if (m != n) {
@@ -438,7 +286,6 @@ void spring_electrical_embedding_fast(int dim, SparseMatrix A0, spring_electrica
   if (K < 0){
     ctrl->K = K = average_edge_length(A, dim, x);
   }
-  if (C < 0) ctrl->C = C = 0.2;
   if (p >= 0) ctrl->p = p = -1;
   KP = pow(K, 1 - p);
   CRK = pow(C, (2.-p)/3.)/K;
@@ -446,10 +293,6 @@ void spring_electrical_embedding_fast(int dim, SparseMatrix A0, spring_electrica
   force = gv_calloc(dim * n, sizeof(double));
 
   do {
-#ifdef TIME
-    //start2 = clock();
-#endif
-
     iter++;
     Fnorm0 = Fnorm;
     Fnorm = 0.;
@@ -459,7 +302,7 @@ void spring_electrical_embedding_fast(int dim, SparseMatrix A0, spring_electrica
 #ifdef TIME
     start = clock();
 #endif
-    qt = QuadTree_new_from_point_list(dim, n, max_qtree_level, x);
+    QuadTree qt = QuadTree_new_from_point_list(dim, n, max_qtree_level, x);
 
 #ifdef TIME
     qtree_new_cpu += ((double) (clock() - start))/CLOCKS_PER_SEC;
@@ -470,7 +313,7 @@ void spring_electrical_embedding_fast(int dim, SparseMatrix A0, spring_electrica
     start = clock();
 #endif
 
-    QuadTree_get_repulsive_force(qt, force, x, ctrl->bh, p, KP, counts);
+    QuadTree_get_repulsive_force(qt, force, x, bh, p, KP, counts);
 
 #ifdef TIME
     end = clock();
@@ -513,36 +356,17 @@ void spring_electrical_embedding_fast(int dim, SparseMatrix A0, spring_electrica
       qtree_new_cpu += ((double) (end - start)) / CLOCKS_PER_SEC;
 #endif
 
-#ifdef TIME
-      qtree_cpu0 = qtree_cpu - qtree_cpu0;
-      qtree_new_cpu0 = qtree_new_cpu - qtree_new_cpu0;
-      /*      if (Verbose) fprintf(stderr, "\r iter=%d cpu=%.2f, quadtree=%.2f quad_force=%.2f other=%.2f counts={%.2f,%.2f,%.2f} step=%f Fnorm=%f nz=%d  K=%f qtree_lev = %d",
-			   iter, ((double) (clock() - start2)) / CLOCKS_PER_SEC, qtree_new_cpu0,
-			   qtree_cpu0,((double) (clock() - start2))/CLOCKS_PER_SEC - qtree_cpu0 - qtree_new_cpu0,
-			   counts[0], counts[1], counts[2],
-			   step, Fnorm, A->nz,K,max_qtree_level);
-      */
-      qtree_cpu0 = qtree_cpu;
-      qtree_new_cpu0 = qtree_new_cpu;
-#endif
-      oned_optimizer_train(qtree_level_optimizer, counts[0]+0.85*counts[1]+3.3*counts[2]);
+      oned_optimizer_train(&qtree_level_optimizer,
+                           counts[0] + 0.85 * counts[1] + 3.3 * counts[2]);
     } else {
       if (Verbose) {
         fprintf(stderr, "\r                iter = %d, step = %f Fnorm = %f nz = %d  K = %f                                  ",iter, step, Fnorm, A->nz,K);
-#ifdef ENERGY
-        fprintf(stderr, "energy = %f\n",spring_electrical_energy(dim, A, x, p, CRK, KP));
-#endif
       }
     }
 
-    step = update_step(adaptive_cooling, step, Fnorm, Fnorm0, cool);
+    step = update_step(adaptive_cooling, step, Fnorm, Fnorm0);
   } while (step > tol && iter < maxiter);
 
-#ifdef DEBUG_PRINT
-  if (Verbose && 0) fputs("\n", stderr);
-#endif
-
-
 #ifdef DEBUG_PRINT
     if (Verbose) {
       fprintf(stderr, "\n iter = %d, step = %f Fnorm = %f nz = %d  K = %f   ",iter, step, Fnorm, A->nz, K);
@@ -558,7 +382,6 @@ void spring_electrical_embedding_fast(int dim, SparseMatrix A0, spring_electrica
 
 
  RETURN:
-  oned_optimizer_delete(qtree_level_optimizer);
   ctrl->max_qtree_level = max_qtree_level;
 
   if (A != A0) SparseMatrix_delete(A);
@@ -571,11 +394,11 @@ static void spring_electrical_embedding_slow(int dim, SparseMatrix A0, spring_el
   SparseMatrix A = A0;
   int m, n;
   int i, j, k;
-  double p = ctrl->p, K = ctrl->K, C = ctrl->C, CRK, tol = ctrl->tol, maxiter = ctrl->maxiter, cool = ctrl->cool, step = ctrl->step, KP;
+  double p = ctrl->p, K = ctrl->K, CRK, maxiter = ctrl->maxiter, step = ctrl->step, KP;
   int *ia = NULL, *ja = NULL;
   double *f = NULL, dist, F, Fnorm = 0, Fnorm0;
   int iter = 0;
-  int adaptive_cooling = ctrl->adaptive_cooling;
+  const bool adaptive_cooling = ctrl->adaptive_cooling;
   double *force;
 #ifdef TIME
   clock_t start, end, start0, start2;
@@ -607,23 +430,10 @@ static void spring_electrical_embedding_slow(int dim, SparseMatrix A0, spring_el
   if (K < 0){
     ctrl->K = K = average_edge_length(A, dim, x);
   }
-  if (C < 0) ctrl->C = C = 0.2;
   if (p >= 0) ctrl->p = p = -1;
   KP = pow(K, 1 - p);
   CRK = pow(C, (2.-p)/3.)/K;
 
-#ifdef DEBUG_0
-  {
-    FILE *f;
-    agxbuf fname = {0};
-    agxbprint(&fname, "/tmp/graph_layout_0_%d", n);
-    f = fopen(agxbuse(&fname), "w");
-    agxbfree(&fname);
-    export_embedding(f, dim, A, x, NULL);
-    fclose(f);
-  }
-#endif
-
   f = gv_calloc(dim, sizeof(double));
   do {
     for (i = 0; i < dim*n; i++) force[i] = 0;
@@ -682,34 +492,9 @@ static void spring_electrical_embedding_slow(int dim, SparseMatrix A0, spring_el
 
     }/* done vertex i */
 
-#ifdef ENERGY
-    if (Verbose) {
-        fprintf(stderr, "\r                iter = %d, step = %f Fnorm = %f nsuper = 0 nz = %d  K = %f                                  ",iter, step, Fnorm,A->nz,K);
-        fprintf(stderr, "energy = %f\n",spring_electrical_energy(dim, A, x, p, CRK, KP));
-    }
-#endif
-
-
-    step = update_step(adaptive_cooling, step, Fnorm, Fnorm0, cool);
+    step = update_step(adaptive_cooling, step, Fnorm, Fnorm0);
   } while (step > tol && iter < maxiter);
 
-#ifdef DEBUG_PRINT
-  if (Verbose && 0) fputs("\n", stderr);
-#endif
-
-#ifdef DEBUG_PRINT_0
-  {
-    FILE *f;
-    agxbuf fname = {0};
-    agxbprint(&fname, "/tmp/graph_layout%d", n);
-    f = fopen(agxbuse(&fname), "w");
-    agxbfree(&fname);
-    export_embedding(f, dim, A, x, NULL);
-    fclose(f);
-  }
-#endif
-
-
 #ifdef DEBUG_PRINT
     if (Verbose) {
       fprintf(stderr, "iter = %d, step = %f Fnorm = %f nsuper = 0 nz = %d  K = %f   ",iter, step, Fnorm, A->nz,K);
@@ -736,31 +521,30 @@ void spring_electrical_embedding(int dim, SparseMatrix A0, spring_electrical_con
   SparseMatrix A = A0;
   int m, n;
   int i, j, k;
-  double p = ctrl->p, K = ctrl->K, C = ctrl->C, CRK, tol = ctrl->tol, maxiter = ctrl->maxiter, cool = ctrl->cool, step = ctrl->step, KP;
+  double p = ctrl->p, K = ctrl->K, CRK, maxiter = ctrl->maxiter, step = ctrl->step, KP;
   int *ia = NULL, *ja = NULL;
   double *f = NULL, dist, F, Fnorm = 0, Fnorm0;
   int iter = 0;
-  int adaptive_cooling = ctrl->adaptive_cooling;
-  QuadTree qt = NULL;
-  int USE_QT = FALSE;
+  const bool adaptive_cooling = ctrl->adaptive_cooling;
+  bool USE_QT = false;
   int nsuper = 0, nsupermax = 10;
   double *center = NULL, *supernode_wgts = NULL, *distances = NULL, nsuper_avg, counts = 0, counts_avg = 0;
 #ifdef TIME
   clock_t start, end, start0, start2;
-  double qtree_cpu = 0, qtree_cpu0 = 0;
+  double qtree_cpu = 0;
   double total_cpu = 0;
   start0 = clock();
 #endif
   int max_qtree_level = ctrl->max_qtree_level;
-  oned_optimizer qtree_level_optimizer = NULL;
+  oned_optimizer qtree_level_optimizer = {0};
 
   if (!A || maxiter <= 0) return;
 
   m = A->m, n = A->n;
   if (n <= 0 || dim <= 0) return;
 
-  if (n >= ctrl->quadtree_size) {
-    USE_QT = TRUE;
+  if (n >= quadtree_size) {
+    USE_QT = true;
     qtree_level_optimizer = oned_optimizer_new(max_qtree_level);
     center = gv_calloc(nsupermax * dim, sizeof(double));
     supernode_wgts = gv_calloc(nsupermax, sizeof(double));
@@ -783,46 +567,20 @@ void spring_electrical_embedding(int dim, SparseMatrix A0, spring_electrical_con
   if (K < 0){
     ctrl->K = K = average_edge_length(A, dim, x);
   }
-  if (C < 0) ctrl->C = C = 0.2;
   if (p >= 0) ctrl->p = p = -1;
   KP = pow(K, 1 - p);
   CRK = pow(C, (2.-p)/3.)/K;
 
-#ifdef DEBUG_0
-  {
-    FILE *f;
-    agxbuf fname = {0};
-    agxbprint(&fname, "/tmp/graph_layout_0_%d", n);
-    f = fopen(agxbuse(&fname), "w");
-    agxbfree(&fname);
-    export_embedding(f, dim, A, x, NULL);
-    fclose(f);
-  }
-#endif
-
   f = gv_calloc(dim, sizeof(double));
   do {
 
-    //#define VIS_MULTILEVEL
-#ifdef VIS_MULTILEVEL
-  {
-    FILE *f;
-    agxbuf fname = {0};
-    static int count = 0;
-    agxbprint(&fname, "/tmp/multilevel_%d", count++);
-    f = fopen(agxbuse(&fname), "w");
-    agxbfree(&fname);
-    export_embedding(f, dim, A, x, NULL);
-    fclose(f);
-  }
-#endif
-
     iter++;
     Fnorm0 = Fnorm;
     Fnorm = 0.;
     nsuper_avg = 0;
     counts_avg = 0;
 
+    QuadTree qt = NULL;
     if (USE_QT) {
 
       max_qtree_level = oned_optimizer_get(qtree_level_optimizer);
@@ -850,7 +608,7 @@ void spring_electrical_embedding(int dim, SparseMatrix A0, spring_electrical_con
 #ifdef TIME
 	start = clock();
 #endif
-	QuadTree_get_supernodes(qt, ctrl->bh, &(x[dim*i]), i, &nsuper, &nsupermax,
+	QuadTree_get_supernodes(qt, bh, &(x[dim*i]), i, &nsuper, &nsupermax,
 				&center, &supernode_wgts, &distances, &counts);
 
 #ifdef TIME
@@ -891,43 +649,12 @@ void spring_electrical_embedding(int dim, SparseMatrix A0, spring_electrical_con
       QuadTree_delete(qt);
       nsuper_avg /= n;
       counts_avg /= n;
-#ifdef TIME
-      qtree_cpu0 = qtree_cpu - qtree_cpu0;
-      if (Verbose && 0) fprintf(stderr, "\n cpu this outer iter = %f, quadtree time = %f other time = %f\n",((double) (clock() - start2)) / CLOCKS_PER_SEC, qtree_cpu0,((double) (clock() - start2))/CLOCKS_PER_SEC - qtree_cpu0);
-      qtree_cpu0 = qtree_cpu;
-#endif
-      if (Verbose & 0) fprintf(stderr, "nsuper_avg=%f, counts_avg = %f 2*nsuper+counts=%f\n",nsuper_avg,counts_avg, 2*nsuper_avg+counts_avg);
-      oned_optimizer_train(qtree_level_optimizer, 5*nsuper_avg + counts_avg);
+      oned_optimizer_train(&qtree_level_optimizer, 5 * nsuper_avg + counts_avg);
     }
 
-#ifdef ENERGY
-    if (Verbose) {
-        fprintf(stderr, "\r                iter = %d, step = %f Fnorm = %f nsuper = %d nz = %d  K = %f                                  ",iter, step, Fnorm, (int) nsuper_avg,A->nz,K);
-        fprintf(stderr, "energy = %f\n",spring_electrical_energy(dim, A, x, p, CRK, KP));
-    }
-#endif
-
-
-    step = update_step(adaptive_cooling, step, Fnorm, Fnorm0, cool);
+    step = update_step(adaptive_cooling, step, Fnorm, Fnorm0);
   } while (step > tol && iter < maxiter);
 
-#ifdef DEBUG_PRINT
-  if (Verbose && 0) fputs("\n", stderr);
-#endif
-
-#ifdef DEBUG_PRINT_0
-  {
-    FILE *f;
-    agxbuf fname = {0};
-    agxbprint(&fname, "/tmp/graph_layout%d", n);
-    f = fopen(agxbuse(&fname), "w");
-    agxbfree(&fname);
-    export_embedding(f, dim, A, x, NULL);
-    fclose(f);
-  }
-#endif
-
-
 #ifdef DEBUG_PRINT
     if (Verbose) {
       if (USE_QT){
@@ -947,7 +674,6 @@ void spring_electrical_embedding(int dim, SparseMatrix A0, spring_electrical_con
 
  RETURN:
   if (USE_QT) {
-    oned_optimizer_delete(qtree_level_optimizer);
     ctrl->max_qtree_level = max_qtree_level;
   }
   if (A != A0) SparseMatrix_delete(A);
@@ -964,26 +690,25 @@ void spring_electrical_spring_embedding(int dim, SparseMatrix A0, SparseMatrix D
   SparseMatrix A = A0;
   int m, n;
   int i, j, k;
-  double p = ctrl->p, K = ctrl->K, C = ctrl->C, CRK, tol = ctrl->tol, maxiter = ctrl->maxiter, cool = ctrl->cool, step = ctrl->step, KP;
+  double p = ctrl->p, K = ctrl->K, CRK, maxiter = ctrl->maxiter, step = ctrl->step, KP;
   int *ia = NULL, *ja = NULL;
   int *id = NULL, *jd = NULL;
   double *d;
   double *xold = NULL;
   double *f = NULL, dist, F, Fnorm = 0, Fnorm0;
   int iter = 0;
-  int adaptive_cooling = ctrl->adaptive_cooling;
-  QuadTree qt = NULL;
-  int USE_QT = FALSE;
+  const bool adaptive_cooling = ctrl->adaptive_cooling;
+  bool USE_QT = false;
   int nsuper = 0, nsupermax = 10;
-  double *center = NULL, *supernode_wgts = NULL, *distances = NULL, nsuper_avg, counts = 0;
+  double *center = NULL, *supernode_wgts = NULL, *distances = NULL, counts = 0;
   int max_qtree_level = 10;
 
   if (!A  || maxiter <= 0) return;
   m = A->m, n = A->n;
   if (n <= 0 || dim <= 0) return;
 
-  if (n >= ctrl->quadtree_size) {
-    USE_QT = TRUE;
+  if (n >= quadtree_size) {
+    USE_QT = true;
     center = gv_calloc(nsupermax * dim, sizeof(double));
     supernode_wgts = gv_calloc(nsupermax, sizeof(double));
     distances = gv_calloc(nsupermax, sizeof(double));
@@ -1008,23 +733,10 @@ void spring_electrical_spring_embedding(int dim, SparseMatrix A0, SparseMatrix D
   if (K < 0){
     ctrl->K = K = average_edge_length(A, dim, x);
   }
-  if (C < 0) ctrl->C = C = 0.2;
   if (p >= 0) ctrl->p = p = -1;
   KP = pow(K, 1 - p);
   CRK = pow(C, (2.-p)/3.)/K;
 
-#ifdef DEBUG_0
-  {
-    FILE *f;
-    agxbuf fname = {0};
-    agxbprint(&fname, "/tmp/graph_layout_0_%d", n);
-    f = fopen(agxbuse(&fname), "w");
-    agxbfree(&fname);
-    export_embedding(f, dim, A, x, NULL);
-    fclose(f);
-  }
-#endif
-
   f = gv_calloc(dim, sizeof(double));
   xold = gv_calloc(dim * n, sizeof(double));
   do {
@@ -1032,8 +744,8 @@ void spring_electrical_spring_embedding(int dim, SparseMatrix A0, SparseMatrix D
     memcpy(xold, x, sizeof(double)*dim*n);
     Fnorm0 = Fnorm;
     Fnorm = 0.;
-    nsuper_avg = 0;
 
+    QuadTree qt = NULL;
     if (USE_QT) {
       qt = QuadTree_new_from_point_list(dim, n, max_qtree_level, x);
     }
@@ -1065,9 +777,8 @@ void spring_electrical_spring_embedding(int dim, SparseMatrix A0, SparseMatrix D
 
       /* repulsive force K^(1 - p)/||x_i-x_j||^(1 - p) (x_i - x_j) */
       if (USE_QT){
-	QuadTree_get_supernodes(qt, ctrl->bh, &(x[dim*i]), i, &nsuper, &nsupermax,
+	QuadTree_get_supernodes(qt, bh, &(x[dim*i]), i, &nsuper, &nsupermax,
 				&center, &supernode_wgts, &distances, &counts);
-	nsuper_avg += nsuper;
 	for (j = 0; j < nsuper; j++){
 	  dist = MAX(distances[j], MINDIST);
 	  for (k = 0; k < dim; k++){
@@ -1097,37 +808,10 @@ void spring_electrical_spring_embedding(int dim, SparseMatrix A0, SparseMatrix D
     }/* done vertex i */
 
     if (qt) QuadTree_delete(qt);
-    nsuper_avg /= n;
-#ifdef DEBUG_PRINT
-    if (Verbose && 0) {
-        fprintf(stderr, "\r                iter = %d, step = %f Fnorm = %f nsuper = %d nz = %d  K = %f                                  ",iter, step, Fnorm, (int) nsuper_avg,A->nz,K);
-#ifdef ENERGY
-        fprintf(stderr, "energy = %f\n",spring_electrical_energy(dim, A, x, p, CRK, KP));
-#endif
-    }
-#else
-    (void)nsuper_avg;
-#endif
 
-    step = update_step(adaptive_cooling, step, Fnorm, Fnorm0, cool);
+    step = update_step(adaptive_cooling, step, Fnorm, Fnorm0);
   } while (step > tol && iter < maxiter);
 
-#ifdef DEBUG_PRINT
-  if (Verbose && 0) fputs("\n", stderr);
-#endif
-
-#ifdef DEBUG_PRINT_0
-  {
-    FILE *f;
-    agxbuf fname = {0};
-    agxbprint(&fname, "/tmp/graph_layout%d", n);
-    f = fopen(agxbuse(&fname), "w");
-    agxbfree(&fname);
-    export_embedding(f, dim, A, x, NULL);
-    fclose(f);
-  }
-#endif
-
   if (ctrl->beautify_leaves) beautify_leaves(dim, A, x);
 
  RETURN:
@@ -1139,25 +823,7 @@ void spring_electrical_spring_embedding(int dim, SparseMatrix A0, SparseMatrix D
   free(distances);
 }
 
-
-
-
-void print_matrix(double *x, int n, int dim){
-  int i, k;
-  printf("{");
-  for (i = 0; i < n; i++){
-    if (i != 0) printf(",");
-    printf("{");
-    for (k = 0; k < dim; k++) {
-      if (k != 0) printf(",");
-      printf("%f",x[i*dim+k]);
-    }
-    printf("}");
-  }
-  printf("}\n");
-}
-
-void interpolate_coord(int dim, SparseMatrix A, double *x){
+static void interpolate_coord(int dim, SparseMatrix A, double *x) {
   int i, j, k, *ia = A->ia, *ja = A->ja, nz;
   double alpha = 0.5, beta;
 
@@ -1182,7 +848,7 @@ void interpolate_coord(int dim, SparseMatrix A, double *x){
 }
 static void prolongate(int dim, SparseMatrix A, SparseMatrix P, SparseMatrix R, double *x, double *y, double delta){
   int nc, *ia, *ja, i, j, k;
-  SparseMatrix_multiply_dense(P, x, &y, dim);
+  SparseMatrix_multiply_dense(P, x, y, dim);
 
   interpolate_coord(dim, A, y);
   nc = R->m;
@@ -1197,11 +863,9 @@ static void prolongate(int dim, SparseMatrix A, SparseMatrix P, SparseMatrix R,
   }
 }
 
-
-
-int power_law_graph(SparseMatrix A){
+static bool power_law_graph(SparseMatrix A) {
   int m, max = 0, i, *ia = A->ia, *ja = A->ja, j, deg;
-  int res = FALSE;
+  bool res = false;
   m = A->m;
   int *mask = gv_calloc(m + 1, sizeof(int));
 
@@ -1218,7 +882,7 @@ int power_law_graph(SparseMatrix A){
     mask[deg]++;
     max = MAX(max, mask[deg]);
   }
-  if (mask[1] > 0.8*max && mask[1] > 0.3*m) res = TRUE;
+  if (mask[1] > 0.8*max && mask[1] > 0.3*m) res = true;
   free(mask);
   return res;
 }
@@ -1400,10 +1064,6 @@ static SparseMatrix shorting_edge_label_nodes(SparseMatrix A, int n_edge_label_n
 	if (ja[jj] != i && mask[ja[jj]] >= 0) {
 	    irn[nz] = mask[i];
 	    jcn[nz++] = mask[ja[jj]];
-	    if (mask[i] == 68 || mask[ja[jj]] == 68){
-	      fprintf(stderr, "%d %d\n",mask[i], mask[ja[jj]]);
-	      mask[i] = mask[i];
-	    }
 	}
       }
     }
@@ -1418,13 +1078,14 @@ static SparseMatrix shorting_edge_label_nodes(SparseMatrix A, int n_edge_label_n
 
 }
 
-static void multilevel_spring_electrical_embedding_core(int dim, SparseMatrix A0, SparseMatrix D0, spring_electrical_control ctrl, double *label_sizes,
-					    double *x, int n_edge_label_nodes, int *edge_label_nodes, int *flag){
+void multilevel_spring_electrical_embedding(int dim, SparseMatrix A0,
+                                            spring_electrical_control ctrl,
+                                            double *label_sizes, double *x,
+                                            int n_edge_label_nodes,
+                                            int *edge_label_nodes, int *flag) {
 
-
-  Multilevel_control mctrl = NULL;
-  int n, plg;
-  SparseMatrix A = A0, D = D0, P = NULL;
+  int n;
+  SparseMatrix A = A0, P = NULL;
   Multilevel grid, grid0;
   double *xc = NULL, *xf = NULL;
   struct spring_electrical_control_struct ctrl0;
@@ -1456,7 +1117,7 @@ static void multilevel_spring_electrical_embedding_core(int dim, SparseMatrix A0
 
     double *x2 = gv_calloc(A->m * dim, sizeof(double));
     A2 = shorting_edge_label_nodes(A, n_edge_label_nodes, edge_label_nodes);
-    multilevel_spring_electrical_embedding(dim, A2, NULL, ctrl, NULL, x2, 0, NULL, flag);
+    multilevel_spring_electrical_embedding(dim, A2, ctrl, NULL, x2, 0, NULL, flag);
 
     assert(!(*flag));
     attach_edge_label_coordinates(dim, A, n_edge_label_nodes, edge_label_nodes, x, x2);
@@ -1469,9 +1130,8 @@ static void multilevel_spring_electrical_embedding_core(int dim, SparseMatrix A0
     return;
   }
 
-  mctrl = Multilevel_control_new();
-  mctrl->maxlevel = ctrl->multilevels;
-  grid0 = Multilevel_new(A, D, mctrl);
+  Multilevel_control mctrl = {.maxlevel = ctrl->multilevels};
+  grid0 = Multilevel_new(A, mctrl);
 
   grid = Multilevel_get_coarsest(grid0);
   if (Multilevel_is_finest(grid)){
@@ -1480,7 +1140,7 @@ static void multilevel_spring_electrical_embedding_core(int dim, SparseMatrix A0
     xc = gv_calloc(grid->n * dim, sizeof(double));
   }
 
-  plg = power_law_graph(A);
+  const bool plg = power_law_graph(A);
   if (ctrl->p == AUTOP){
     ctrl->p = -1;
     if (plg) ctrl->p = -1.8;
@@ -1522,9 +1182,9 @@ static void multilevel_spring_electrical_embedding_core(int dim, SparseMatrix A0
     prolongate(dim, grid->A, P, grid->R, xc, xf, (ctrl->K)*0.001);
     free(xc);
     xc = xf;
-    ctrl->random_start = FALSE;
+    ctrl->random_start = false;
     ctrl->K = ctrl->K * 0.75;
-    ctrl->adaptive_cooling = FALSE;
+    ctrl->adaptive_cooling = false;
     ctrl->step = .1;
   } while (grid);
 
@@ -1551,12 +1211,5 @@ static void multilevel_spring_electrical_embedding_core(int dim, SparseMatrix A0
  RETURN:
   *ctrl = ctrl0;
   if (A != A0) SparseMatrix_delete(A);
-  if (D && D != D0) SparseMatrix_delete(D);
-  Multilevel_control_delete(mctrl);
   Multilevel_delete(grid0);
-}
-
-void multilevel_spring_electrical_embedding(int dim, SparseMatrix A, SparseMatrix D, spring_electrical_control ctrl, double *label_sizes,
-				 double *x, int n_edge_label_nodes, int *edge_label_nodes, int *flag){
-  multilevel_spring_electrical_embedding_core(dim, A, D, ctrl, label_sizes, x, n_edge_label_nodes, edge_label_nodes, flag);
-}
+ }
