@@ -26,16 +26,10 @@ enum {QUAD_TREE_NONE = 0, QUAD_TREE_NORMAL, QUAD_TREE_FAST, QUAD_TREE_HYBRID};
 
 struct spring_electrical_control_struct {
   double p;/*a negativve real number default to -1. repulsive force = dist^p */
-  double q;/*a positive real number default to 2. attractive force = dist^q */
   double K;/* the natural distance. If K < 0, K will be set to the average distance of an edge */
-  double C;/* another parameter. f_a(i,j) = C*dist(i,j)^2/K * d_ij, f_r(i,j) = K^(3-p)/dist(i,j)^(-p). By default C = 0.2. */
   int multilevels;/* if <=1, single level */
-  int quadtree_size;/* cut off size above which quadtree approximation is used */
   int max_qtree_level;/* max level of quadtree */
-  double bh;/* Barnes-Hutt constant, if width(snode)/dist[i,snode] < bh, treat snode as a supernode. default 0.2*/
-  double tol;/* minimum different between two subsequence config before terminating. ||x-xold|| < tol */
   int maxiter;
-  double cool;/* default 0.9 */
   double step;/* initial step size */
   int random_seed;
   bool random_start : 1; ///< whether to apply SE from a random layout, or from exisiting layout
@@ -70,7 +64,6 @@ void multilevel_spring_electrical_embedding(int dim, SparseMatrix A0,
                                             int n_edge_label_nodes,
                                             int *edge_label_nodes, int *flag);
 
-void export_embedding(FILE *fp, int dim, SparseMatrix A, double *x, double *width);
 void spring_electrical_control_delete(spring_electrical_control ctrl);
 
 double average_edge_length(SparseMatrix A, int dim, double *coord);
