@@ -3,11 +3,12 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
+#include	<limits.h>
 #include	<sfio/sfhdr.h>
 
 /*	Convert a floating point value to ASCII
@@ -32,33 +33,29 @@ char *_sfcvt(void * dv, int n_digit, int *decpt, int *sign, int format)
     char *sp;
     long n, v;
     char *ep, *buf, *endsp;
-    static char *Buf;
-
-    /* set up local buffer */
-    if (!Buf && !(Buf = malloc(SF_MAXDIGITS)))
-	return SF_INFINITE;
+    static char Buf[SF_MAXDIGITS];
 
     *sign = *decpt = 0;
     {
-	double dval = *((double *) dv);
+	double dval = *(double *)dv;
 
 	if (dval == 0.)
 	    return SF_ZERO;
-	else if ((*sign = (dval < 0.)))	/* assignment = */
+	else if ((*sign = dval < 0.))	/* assignment = */
 	    dval = -dval;
 
 	n = 0;
-	if (dval >= (double) SF_MAXLONG) {	/* scale to a small enough number to fit an int */
+	if (dval >= (double)LONG_MAX) {	/* scale to a small enough number to fit an int */
 	    v = SF_MAXEXP10 - 1;
 	    do {
 		if (dval < _Sfpos10[v])
 		    v -= 1;
 		else {
 		    dval *= _Sfneg10[v];
-		    if ((n += (1 << v)) >= SF_IDIGITS)
+		    if ((n += 1 << v) >= SF_IDIGITS)
 			return SF_INFINITE;
 		}
-	    } while (dval >= (double) SF_MAXLONG);
+	    } while (dval >= (double)LONG_MAX);
 	}
 	*decpt = (int) n;
 
@@ -82,7 +79,7 @@ char *_sfcvt(void * dv, int n_digit, int *decpt, int *sign, int format)
 	if (n_digit > 0)
 	    n += n_digit;
 
-	if ((ep = (sp + n)) > (endsp = Buf + (SF_MAXDIGITS - 2)))
+	if ((ep = sp + n) > (endsp = Buf + (SF_MAXDIGITS - 2)))
 	    ep = endsp;
 	if (sp > ep)
 	    sp = ep;
