@@ -3,12 +3,14 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #include <assert.h>
+#include <cgraph/gv_ctype.h>
+#include <cgraph/gv_math.h>
 #include <inttypes.h>
 #include	<sfio/sfhdr.h>
 #include	<stddef.h>
@@ -21,20 +23,19 @@
 */
 
 #define HIGHBITI	(~((~0u) >> 1))
-#define HIGHBITL	(~((~((Sfulong_t)0)) >> 1))
+#define HIGHBITL	(~((~0ull) >> 1))
 
 #define SFFMT_PREFIX	(SFFMT_MINUS|SFFMT_SIGN|SFFMT_BLANK)
 
 #define FPRECIS		6	/* default precision for floats         */
 
-
 /**
  * @param f file to print to
  * @param format Structure describing how to print
  */
 int sfprint(FILE *f, Sffmt_t *format) {
     int v = 0, n_s, base, fmt, flags;
-    Sflong_t lv;
+    long long lv;
     char *sp, *ssp, *endsp, *ep, *endep;
     int dot, width, precis, n, n_output = 0;
     int sign, decpt;
@@ -46,9 +47,7 @@ int sfprint(FILE *f, Sffmt_t *format) {
 
     Argv_t argv;		/* for extf to return value     */
     Sffmt_t *ft;		/* format environment           */
-    Fmt_t *fm, *fmstk;		/* stack contexts               */
 
-    Fmtpos_t *fp;		/* arg position list            */
     int argp, argn;		/* arg position and number      */
 
 #define SLACK		1024
@@ -69,43 +68,26 @@ int sfprint(FILE *f, Sffmt_t *format) {
     (n) = 0; \
 } while (0)
 #define SFwrite(f,s,n) do { \
-  if (fwrite((s), (n), 1, (f)) < 1) { \
+  if ((n) > 0 && fwrite((s), (size_t)(n), 1, (f)) < 1) { \
     goto done; \
   } \
 } while (0)
 
     tls[1] = NULL;
 
-    fmstk = NULL;
     ft = NULL;
 
-    fp = NULL;
     argn = -1;
 
     // stack a new environment
     argv.ft = format;
     assert(argv.ft != NULL);
     assert(argv.ft->form != NULL);
-    if (!(fm = malloc(sizeof(Fmt_t))))
-	goto done;
-
-    fm->form = "";
-    memset(&fm->args, 0, sizeof(fm->args));
-
-    fm->argn = argn;
-    fm->fp = NULL;
 
     const char *form = argv.ft->form;
-    va_list args;
-    va_copy(args, argv.ft->args);
-    fp = NULL;
 
-    fm->ft = ft;
-    fm->next = fmstk;
-    fmstk = fm;
     ft = argv.ft;
 
-  loop_fmt:
     while ((n = *form)) {
 	if (n != '%') {		/* collect the non-pattern chars */
 	    sp = (char *) form++;
@@ -129,7 +111,7 @@ int sfprint(FILE *f, Sffmt_t *format) {
 	switch ((fmt = *form++)) {
 	case '\0':
 	    SFputc(f, '%');
-	    goto pop_fmt;
+	    goto done;
 	case '%':
 	    SFputc(f, '%');
 	    continue;
@@ -153,27 +135,15 @@ int sfprint(FILE *f, Sffmt_t *format) {
 			n_str = (form - 1) - t_str;
 		    else {
 			t_str = _Sffmtintf(t_str + 1, &n);
-			n = FP_SET(-1, argn);
+			FP_SET(-1, argn);
 
-			if (fp) {
-			    t_str = fp[n].argv.s;
-			    n_str = fp[n].ft.size;
-			} else if (ft && ft->extf) {
-			    FMTSET(ft, form, args,
-				   LEFTP, 0, 0, 0, 0, 0, NULL, 0);
-			    n = ft->extf(&argv, ft);
-			    if (n < 0)
-				goto pop_fmt;
-			    if (!(ft->flags & SFFMT_VALUE))
-				goto t_arg;
-			    if ((t_str = argv.s) &&
-				(n_str = (int) ft->size) < 0)
-				n_str = (ssize_t)strlen(t_str);
-			} else {
-			  t_arg:
-			    if ((t_str = va_arg(args, char *)))
-				 n_str = (ssize_t)strlen(t_str);
-			}
+			FMTSET(ft, form, LEFTP, 0, 0, 0, 0, 0, NULL, 0);
+			n = ft->extf(&argv, ft);
+			if (n < 0)
+			    goto done;
+			assert(ft->flags & SFFMT_VALUE);
+			if ((t_str = argv.s) && (n_str = (int)ft->size) < 0)
+			    n_str = (ssize_t)strlen(t_str);
 		    }
 		    goto loop_flags;
 		default:
@@ -214,7 +184,7 @@ int sfprint(FILE *f, Sffmt_t *format) {
 		base = 0;	/* for %s,%c */
 		if (*form == 'c' || *form == 's')
 		    goto loop_flags;
-		if (*form && !isalnum((int)*form) &&
+		if (*form && !gv_isalnum(*form) &&
 		    (form[1] == 'c' || form[1] == 's')) {
 		    if (*form == '*')
 			goto do_star;
@@ -225,7 +195,7 @@ int sfprint(FILE *f, Sffmt_t *format) {
 		}
 	    }
 
-	    if (isdigit((int)*form)) {
+	    if (gv_isdigit(*form)) {
 		fmt = *form++;
 		goto dot_size;
 	    } else if (*form != '*')
@@ -236,19 +206,11 @@ int sfprint(FILE *f, Sffmt_t *format) {
 	    form = _Sffmtintf(form, &n);
 	    n = FP_SET(-1, argn);
 
-	    if (fp)
-		v = fp[n].argv.i;
-	    else if (ft && ft->extf) {
-		FMTSET(ft, form, args, '.', dot, 0, 0, 0, 0, NULL,
-		       0);
-		if (ft->extf(&argv, ft) < 0)
-		    goto pop_fmt;
-		if (ft->flags & SFFMT_VALUE)
-		    v = argv.i;
-		else
-		    v = (dot <= 2) ? va_arg(args, int) : 0;
-	    } else
-		v = dot <= 2 ? va_arg(args, int) : 0;
+	    FMTSET(ft, form, '.', dot, 0, 0, 0, 0, NULL, 0);
+	    if (ft->extf(&argv, ft) < 0)
+		goto done;
+	    assert(ft->flags & SFFMT_VALUE);
+	    v = argv.i;
 	    goto dot_set;
 
 	case '1':
@@ -261,7 +223,7 @@ int sfprint(FILE *f, Sffmt_t *format) {
 	case '8':
 	case '9':
 	  dot_size:
-	    for (v = fmt - '0'; isdigit((int)*form); ++form)
+	    for (v = fmt - '0'; gv_isdigit(*form); ++form)
 		v = v * 10 + (*form - '0');
 	  dot_set:
 	    if (dot == 0) {
@@ -278,26 +240,18 @@ int sfprint(FILE *f, Sffmt_t *format) {
 	case 'I':		/* object length */
 	    size = 0;
 	    flags = (flags & ~SFFMT_TYPES) | SFFMT_IFLAG;
-	    if (isdigit((int)*form)) {
-		for (n = *form; isdigit(n); n = *++form)
+	    if (gv_isdigit(*form)) {
+		for (n = *form; gv_isdigit(n); n = *++form)
 		    size = size * 10 + (n - '0');
 	    } else if (*form == '*') {
 		form = _Sffmtintf(form + 1, &n);
 		n = FP_SET(-1, argn);
 
-		if (fp)		/* use position list */
-		    size = fp[n].argv.i;
-		else if (ft && ft->extf) {
-		    FMTSET(ft, form, args, 'I', sizeof(int), 0, 0, 0, 0,
-			   NULL, 0);
-		    if (ft->extf(&argv, ft) < 0)
-			goto pop_fmt;
-		    if (ft->flags & SFFMT_VALUE)
-			size = argv.i;
-		    else
-			size = va_arg(args, int);
-		} else
-		    size = va_arg(args, int);
+		FMTSET(ft, form, 'I', sizeof(int), 0, 0, 0, 0, NULL, 0);
+		if (ft->extf(&argv, ft) < 0)
+		    goto done;
+		assert(ft->flags & SFFMT_VALUE);
+		size = argv.i;
 	    }
 	    goto loop_flags;
 
@@ -336,72 +290,38 @@ int sfprint(FILE *f, Sffmt_t *format) {
 	    size = -1;
 	    flags = (flags & ~SFFMT_TYPES) | SFFMT_TFLAG;
 	    goto loop_flags;
+	default: // continue with logic below
+	    break;
 	}
 
 	/* set the correct size */
 	if (flags & (SFFMT_TYPES & ~SFFMT_IFLAG)) {
 	    if (_Sftype[fmt] & (SFFMT_INT | SFFMT_UINT)) {
-		size = (flags & SFFMT_LLONG) ? sizeof(Sflong_t) :
-		    (flags & SFFMT_LONG) ? sizeof(long) :
-		    (flags & SFFMT_SHORT) ? sizeof(short) :
-		    (flags & SFFMT_SSHORT) ? sizeof(char) :
-		    (flags & SFFMT_JFLAG) ? sizeof(Sflong_t) :
-		    (flags & SFFMT_TFLAG) ? sizeof(ptrdiff_t) :
-		    (flags & SFFMT_ZFLAG) ? sizeof(size_t) : -1;
+		size = (flags & SFFMT_LLONG)  ? (ssize_t)sizeof(long long) :
+		       (flags & SFFMT_LONG)   ? (ssize_t)sizeof(long) :
+		       (flags & SFFMT_SHORT)  ? (ssize_t)sizeof(short) :
+		       (flags & SFFMT_SSHORT) ? (ssize_t)sizeof(char) :
+		       (flags & SFFMT_JFLAG)  ? (ssize_t)sizeof(long long) :
+		       (flags & SFFMT_TFLAG)  ? (ssize_t)sizeof(ptrdiff_t) :
+		       (flags & SFFMT_ZFLAG)  ? (ssize_t)sizeof(size_t) : -1;
 	    } else if (_Sftype[fmt] & SFFMT_FLOAT) {
-		size = (flags & SFFMT_LDOUBLE) ? sizeof(Sfdouble_t) :
-		    (flags & (SFFMT_LONG | SFFMT_LLONG)) ?
-		    sizeof(double) : -1;
+		size = (flags & SFFMT_LDOUBLE)              ? (ssize_t)sizeof(long double) :
+		       (flags & (SFFMT_LONG | SFFMT_LLONG)) ? (ssize_t)sizeof(double) : -1;
 	    }
 	}
 
 	argp = FP_SET(argp, argn);
-	if (fp) {
-	    if (ft && ft->extf && fp[argp].ft.fmt != fp[argp].fmt)
-		fmt = fp[argp].ft.fmt;
-	    argv = fp[argp].argv;
-	    size = fp[argp].ft.size;
-	} else if (ft && ft->extf) {	/* extended processing */
-	    FMTSET(ft, form, args, fmt, size, flags, width, precis, base,
-		   t_str, n_str);
-	    v = ft->extf(&argv, ft);
+	FMTSET(ft, form, fmt, size, flags, width, precis, base, t_str, n_str);
+	v = ft->extf(&argv, ft);
 
-	    if (v < 0)
-		goto pop_fmt;
-	    else if (v == 0) {	/* extf did not output */
-		FMTGET(ft, form, args, fmt, size, flags, width, precis,
-		       base);
-		if (!(ft->flags & SFFMT_VALUE))
-		    goto get_val;
-	    } else if (v > 0) {	/* extf output v bytes */
-		n_output += v;
-		continue;
-	    }
-	} else {
-	  get_val:
-	    switch (_Sftype[fmt]) {
-	    case SFFMT_INT:
-	    case SFFMT_UINT:
-		if (FMTCMP(size, long, Sflong_t))
-		     argv.l = va_arg(args, long);
-		else
-		    argv.i = va_arg(args, int);
-		break;
-	    case SFFMT_FLOAT:
-		    argv.d = va_arg(args, double);
-		break;
-	    case SFFMT_POINTER:
-		argv.vp = va_arg(args, void *);
-		break;
-	    case SFFMT_BYTE:
-		if (base >= 0)
-		    argv.s = va_arg(args, char *);
-		else
-		    argv.c = (char) va_arg(args, int);
-		break;
-	    default:		/* unknown pattern */
-		break;
-	    }
+	if (v < 0)
+	    goto done;
+	else if (v == 0) { // extf did not output
+	    FMTGET(ft, form, fmt, size, flags, width, precis, base);
+	    assert(ft->flags & SFFMT_VALUE);
+	} else if (v > 0) { // extf output v bytes
+	    n_output += v;
+	    continue;
 	}
 
 	switch (fmt) {		/* PRINTF DIRECTIVES */
@@ -488,7 +408,7 @@ int sfprint(FILE *f, Sffmt_t *format) {
 		(flags & ~(SFFMT_SIGN | SFFMT_BLANK | SFFMT_ZERO)) |
 		SFFMT_ALTER;
 	    if (sizeof(void*) > sizeof(int)) {
-		lv = (Sflong_t)(intptr_t)argv.vp;
+		lv = (long long)(intptr_t)argv.vp;
 		goto long_cvt;
 	    } else {
 		v = (int)(intptr_t)argv.vp;
@@ -531,43 +451,42 @@ int sfprint(FILE *f, Sffmt_t *format) {
 
 	  int_arg:
 	    if ((sizeof(long) > sizeof(int) || sizeof(void*) > sizeof(int))
-	        && FMTCMP(size, Sflong_t, Sflong_t)) {
+	        && FMTCMP(size, long long, long long)) {
 		lv = argv.ll;
 		goto long_cvt;
 	    } else if ((sizeof(long) > sizeof(int) || sizeof(void*) > sizeof(int))
-	               && FMTCMP(size, long, Sflong_t)) {
+	               && FMTCMP(size, long, long long)) {
 		if (fmt == 'd')
-		    lv = (Sflong_t) argv.l;
+		    lv = (long long)argv.l;
 		else
-		    lv = (Sflong_t) ((ulong) argv.l);
+		    lv = (long long)(ulong)argv.l;
 	      long_cvt:
 		if (lv == 0 && precis == 0)
 		    break;
 		if (lv < 0 && fmt == 'd') {
 		    flags |= SFFMT_MINUS;
-		    if (lv == HIGHBITL) {	/* avoid overflow */
-			lv = (Sflong_t) (HIGHBITL / base);
+		    if ((unsigned long long)lv == HIGHBITL) { // avoid overflow
+			lv = (long long)(HIGHBITL / (unsigned long long)base);
 			*--sp = _Sfdigits[HIGHBITL -
-					  ((Sfulong_t) lv) * base];
+			                  (unsigned long long)lv * (unsigned long long)base];
 		    } else
 			lv = -lv;
 		}
 		if (n_s < 0) {	/* base 10 */
-		    Sflong_t nv;
-		    sfucvt(lv, sp, nv, ssp, Sflong_t, Sfulong_t);
+		    long long nv;
+		    sfucvt(lv, sp, nv, ssp, long long, unsigned long long);
 		} else if (n_s > 0) {	/* base power-of-2 */
 		    do {
 			*--sp = ssp[lv & n_s];
-		    } while ((lv = ((Sfulong_t) lv) >> n));
+		    } while ((lv = (unsigned long long)lv >> n));
 		} else {	/* general base */
 		    do {
-			*--sp = ssp[((Sfulong_t) lv) % base];
-		    } while ((lv = ((Sfulong_t) lv) / base));
+			*--sp = ssp[(unsigned long long)lv % (unsigned long long)base];
+		    } while ((lv = (unsigned long long)lv / (unsigned long long)base));
 		}
 	    } else
-	    if (sizeof(short) < sizeof(int)
-		    && FMTCMP(size, short, Sflong_t)) {
-		if (ft && ft->extf && (ft->flags & SFFMT_VALUE)) {
+	    if (sizeof(short) < sizeof(int) && FMTCMP(size, short, long long)) {
+		if (ft->flags & SFFMT_VALUE) {
 		    if (fmt == 'd')
 			v = (int) ((short) argv.h);
 		    else
@@ -580,7 +499,7 @@ int sfprint(FILE *f, Sffmt_t *format) {
 		}
 		goto int_cvt;
 	    } else if (size == sizeof(char)) {
-		if (ft && ft->extf && (ft->flags & SFFMT_VALUE)) {
+		if (ft->flags & SFFMT_VALUE) {
 		    if (fmt == 'd')
 			v = (int) ((char) argv.c);
 		    else
@@ -599,9 +518,9 @@ int sfprint(FILE *f, Sffmt_t *format) {
 		    break;
 		if (v < 0 && fmt == 'd') {
 		    flags |= SFFMT_MINUS;
-		    if (v == HIGHBITI) {	/* avoid overflow */
-			v = (int) (HIGHBITI / base);
-			*--sp = _Sfdigits[HIGHBITI - (unsigned)v * base];
+		    if ((unsigned)v == HIGHBITI) { // avoid overflow
+			v = (int)(HIGHBITI / (unsigned)base);
+			*--sp = _Sfdigits[HIGHBITI - (unsigned)v * (unsigned)base];
 		    } else
 			v = -v;
 		}
@@ -613,8 +532,8 @@ int sfprint(FILE *f, Sffmt_t *format) {
 		    } while ((v = (unsigned)v >> n));
 		} else {	/* n_s == 0, general base */
 		    do {
-			*--sp = ssp[(unsigned)v % base];
-		    } while ((v = (unsigned)v / base));
+			*--sp = ssp[(unsigned)v % (unsigned)base];
+		    } while ((v = (unsigned)v / (unsigned)base));
 		}
 	    }
 
@@ -679,8 +598,7 @@ int sfprint(FILE *f, Sffmt_t *format) {
 	case 'e':
 	case 'E':
 	case 'f':
-	    if (!(ft && ft->extf && (ft->flags & SFFMT_VALUE)) ||
-		    FMTCMP(size, double, Sfdouble_t))
+	    if (!(ft->flags & SFFMT_VALUE) || FMTCMP(size, double, long double))
 		 dval = argv.d;
 	    else
 		dval = (double) argv.f;
@@ -688,14 +606,14 @@ int sfprint(FILE *f, Sffmt_t *format) {
 	    if (fmt == 'e' || fmt == 'E') {
 		n = (precis = precis < 0 ? FPRECIS : precis) + 1;
 		{
-		    ep = _sfcvt(&dval, min(n, SF_FDIGITS),
+		    ep = _sfcvt(&dval, imin(n, SF_FDIGITS),
 				&decpt, &sign, SFFMT_EFORMAT);
 		}
 		goto e_format;
 	    } else if (fmt == 'f' || fmt == 'F') {
 		precis = precis < 0 ? FPRECIS : precis;
 		{
-		    ep = _sfcvt(&dval, min(precis, SF_FDIGITS),
+		    ep = _sfcvt(&dval, imin(precis, SF_FDIGITS),
 				&decpt, &sign, 0);
 		}
 		goto f_format;
@@ -704,7 +622,7 @@ int sfprint(FILE *f, Sffmt_t *format) {
 	    /* 'g' or 'G' format */
 	    precis = precis < 0 ? FPRECIS : precis == 0 ? 1 : precis;
 	    {
-		ep = _sfcvt(&dval, min(precis, SF_FDIGITS),
+		ep = _sfcvt(&dval, imin(precis, SF_FDIGITS),
 			    &decpt, &sign, SFFMT_EFORMAT);
 		if (dval == 0.)
 		    decpt = 1;
@@ -729,7 +647,7 @@ int sfprint(FILE *f, Sffmt_t *format) {
 	    }
 
 	  e_format:		/* build the x.yyyy string */
-	    if (isalpha((int)*ep))
+	    if (gv_isalpha(*ep))
 		goto infinite;
 	    sp = endsp = buf + 1;	/* reserve space for sign */
 	    *endsp++ = *ep ? *ep++ : '0';
@@ -760,12 +678,12 @@ int sfprint(FILE *f, Sffmt_t *format) {
 
 	    /* the e/Exponent separator and sign */
 	    *--ep = (decpt > 0 || dval == 0.) ? '+' : '-';
-	    *--ep = isupper(fmt) ? 'E' : 'e';
+	    *--ep = gv_isupper(fmt) ? 'E' : 'e';
 
 	    goto end_efg;
 
 	  f_format:		/* data before the decimal point */
-	    if (isalpha((int)*ep)) {
+	    if (gv_isalpha(*ep)) {
 	      infinite:
 		endsp = (sp = ep) + sfslen();
 		ep = endep;
@@ -795,7 +713,7 @@ int sfprint(FILE *f, Sffmt_t *format) {
 		*endsp++ = decimal;
 
 	    if ((n = -decpt) > 0) {	/* output zeros for negative exponent */
-		ssp = endsp + min(n, precis);
+		ssp = endsp + imin(n, precis);
 		precis -= n;
 		while (endsp < ssp)
 		    *endsp++ = '0';
@@ -868,28 +786,7 @@ int sfprint(FILE *f, Sffmt_t *format) {
 	}
     }
 
-  pop_fmt:
-    free(fp);
-    fp = NULL;
-    while ((fm = fmstk)) {	/* pop the format stack and continue */
-	fmstk = fm->next;
-	if ((form = fm->form)) {
-	    va_copy(args, fm->args);
-	    argn = fm->argn;
-	    fp = fm->fp;
-	}
-	ft = fm->ft;
-	free(fm);
-	if (form && form[0])
-	    goto loop_fmt;
-    }
-
   done:
-    free(fp);
-    while ((fm = fmstk)) {
-	fmstk = fm->next;
-	free(fm);
-    }
 
     return n_output;
 }
