@@ -9,8 +9,6 @@
  *************************************************************************/
 
 #define STANDALONE
-#include <cgraph/agxbuf.h>
-#include <cgraph/startswith.h>
 #include <sparse/general.h>
 #include <sparse/DotIO.h>
 #include <sparse/clustering.h>
@@ -20,7 +18,9 @@
 #include <sparse/colorutil.h>
 #include <stdbool.h>
 #include <string.h>
+#include <util/agxbuf.h>
 #include <util/alloc.h>
+#include <util/startswith.h>
 #include <util/unreachable.h>
 
 typedef struct {
