@@ -3,12 +3,11 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-#include <vmalloc/vmhdr.h>
 #include <vmalloc/vmalloc.h>
 #include <stdlib.h>
 
@@ -23,9 +22,5 @@ Vmalloc_t *vmopen(void) {
     return NULL;
   }
 
-  vm->meth.allocf = bestalloc;
-  vm->meth.resizef = bestresize;
-  vm->meth.freef = bestfree;
-
   return vm;
 }
