@@ -29,17 +29,17 @@ public:
 	Variable *right;
 	double gap;
 	double lm;
-	Constraint(Variable *left, Variable *right, double gap, bool equality=false);
+	Constraint(Variable *left, Variable *right, double gap);
 	~Constraint();
 	double slack() const { return right->position() - gap - left->position(); }
 	long timeStamp;
 	bool active;
 	bool visited;
-	bool equality;
 };
 #include <float.h>
 #include <vpsc/block.h>
-static inline bool compareConstraints(Constraint *const &l, Constraint *const &r) {
+static inline bool compareConstraints(const Constraint *const l,
+                                      const Constraint *const r) {
 	double const sl = 
 		l->left->block->timeStamp > l->timeStamp
 		||l->left->block==l->right->block
