@@ -3,19 +3,19 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-#include <cgraph/agxbuf.h>
-#include <cgraph/alloc.h>
 #include <cgraph/gv_ctype.h>
-#include <cgraph/prisize_t.h>
-#include <cgraph/unreachable.h>
 #include <xdot/xdot.h>
 #include <stdlib.h>
 #include <string.h>
+#include <util/agxbuf.h>
+#include <util/alloc.h>
+#include <util/prisize_t.h>
+#include <util/unreachable.h>
 
 /* the parse functions should return NULL on error */
 static char *parseReal(char *s, double *fp)
@@ -586,6 +586,8 @@ static void printXDot_Op(xdot_op * op, pf print, void *info, int more)
 	printRect(&op->u.image.pos, print, info);
 	printString(op->u.image.name, print, info);
 	break;
+    default: // invalid type; ignore
+	break;
     }
     if (more)
 	print(info, " ");
@@ -699,6 +701,8 @@ static void jsonXDot_Op(xdot_op * op, pf print, void *info, int more)
 	jsonString(op->u.image.name, print, info);
 	print(info, "]");
 	break;
+    default: // invalid type; ignore
+	break;
     }
     if (more)
 	print(info, "},\n");
