@@ -3,16 +3,18 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #include "config.h"
 
+#include <assert.h>
+#include <stdbool.h>
 #include <stdlib.h>
-#include <stddef.h>
 #include <string.h>
+#include <util/alloc.h>
 
 #ifdef HAVE_PANGOCAIRO
 #include <cairo.h>
@@ -31,7 +33,7 @@ typedef enum {
 
 static void gd_freeimage(usershape_t *us)
 {
-    gdImageDestroy((gdImagePtr)us->data);
+    gdImageDestroy(us->data);
 }
 
 static gdImagePtr gd_loadimage(GVJ_t * job, usershape_t *us)
@@ -51,40 +53,20 @@ static gdImagePtr gd_loadimage(GVJ_t * job, usershape_t *us)
 	if (!gvusershape_file_access(us))
 	    return NULL;
 	switch (us->type) {
-#if 0
-	    case FT_GD:
-		im = gdImageCreateFromGd(us->f);
-		break;
-	    case FT_GD2:
-		im = gdImageCreateFromGd2(us->f);
-		break;
-#endif
 #ifdef HAVE_GD_PNG
 	    case FT_PNG:
-		us->data = (void*)gdImageCreateFromPng(us->f);
+		us->data = gdImageCreateFromPng(us->f);
 		break;
 #endif
 #ifdef HAVE_GD_GIF
 	    case FT_GIF:
-		us->data = (void*)gdImageCreateFromGif(us->f);
+		us->data = gdImageCreateFromGif(us->f);
 		break;
 #endif
 #ifdef HAVE_GD_JPEG
 	    case FT_JPEG:
-		us->data = (void*)gdImageCreateFromJpeg(us->f);
+		us->data = gdImageCreateFromJpeg(us->f);
 		break;
-#endif
-#if 0
-#ifdef HAVE_GD_XPM
-	    case FT_XPM:
-		us->data = (void*)gdImageCreateFromXpm(us->f);
-		break;
-#endif
-#ifdef HAVE_GD_WBMP
-	    case FT_WBMP:
-		us->data = (void*)gdImageCreateFromWbmp(us->f);
-		break;
-#endif
 #endif
 	    default:
 		break;
@@ -94,7 +76,7 @@ static gdImagePtr gd_loadimage(GVJ_t * job, usershape_t *us)
 
 	gvusershape_file_release(us);
     }
-    return (gdImagePtr)(us->data);
+    return us->data;
 }
 
 static gdImagePtr gd_rotateimage(gdImagePtr im, int rotation)
@@ -107,9 +89,11 @@ static gdImagePtr gd_rotateimage(gdImagePtr im, int rotation)
     return im2;
 }
 	
-static void gd_loadimage_gd(GVJ_t * job, usershape_t *us, boxf b, boolean filled)
+static void gd_loadimage_gd(GVJ_t * job, usershape_t *us, boxf b, bool filled)
 {
-    gdImagePtr im2, im = (gdImagePtr) job->context;
+    (void)filled;
+
+    gdImagePtr im2, im = job->context;
 
     if ((im2 = gd_loadimage(job, us))) {
         if (job->rotation)
@@ -120,21 +104,23 @@ static void gd_loadimage_gd(GVJ_t * job, usershape_t *us, boxf b, boolean filled
 }
 
 #ifdef HAVE_PANGOCAIRO
-static void gd_loadimage_cairo(GVJ_t * job, usershape_t *us, boxf b, boolean filled)
+static void gd_loadimage_cairo(GVJ_t * job, usershape_t *us, boxf b, bool filled)
 {
-    cairo_t *cr = (cairo_t *) job->context; /* target context */
-    unsigned int x, y, stride, width, height, px;
-    unsigned char *data;
+    (void)filled;
+
+    cairo_t *cr = job->context; /* target context */
+    int x, y, width, height;
     cairo_surface_t *surface;    /* source surface */
     gdImagePtr im;
 
     if ((im = gd_loadimage(job, us))) {
 	width = im->sx;
 	height = im->sy;
-// cairo_format_stride_for_width() not available prior to cairo-1.6.4 or so (fc9)
-//stride = cairo_format_stride_for_width (CAIRO_FORMAT_ARGB32, width);
-	stride = width*4;
-	data = malloc (stride * height);
+        const int stride = cairo_format_stride_for_width(CAIRO_FORMAT_ARGB32,
+                                                         width);
+	assert(stride >= 0);
+	assert(height >= 0);
+	unsigned char *data = gv_calloc((size_t)stride, (size_t)height);
 	surface = cairo_image_surface_create_for_data (data, CAIRO_FORMAT_ARGB32,
 							width, height, stride);
 
@@ -142,21 +128,22 @@ static void gd_loadimage_cairo(GVJ_t * job, usershape_t *us, boxf b, boolean fil
 	    if (im->saveAlphaFlag) {
 	        for (y = 0; y < height; y++) {
 		    for (x = 0; x < width; x++) {
-		        px = gdImageTrueColorPixel(im, x, y);
-		        *data++ = gdTrueColorGetBlue(px);
-		        *data++ = gdTrueColorGetGreen(px);
-		        *data++ = gdTrueColorGetRed(px);
-		        *data++ = (0x7F-gdTrueColorGetAlpha(px)) << 1;
+		        const int px = gdImageTrueColorPixel(im, x, y);
+		        *data++ = (unsigned char)gdTrueColorGetBlue(px);
+		        *data++ = (unsigned char)gdTrueColorGetGreen(px);
+		        *data++ = (unsigned char)gdTrueColorGetRed(px);
+		        // gd’s alpha is 7-bit, so scale up ×2 to our 8-bit
+		        *data++ = (unsigned char)((0x7F - gdTrueColorGetAlpha(px)) << 1);
 		    }
 		}
 	    }
 	    else {
 	        for (y = 0; y < height; y++) {
 		    for (x = 0; x < width; x++) {
-		        px = gdImageTrueColorPixel(im, x, y);
-		        *data++ = gdTrueColorGetBlue(px);
-		        *data++ = gdTrueColorGetGreen(px);
-		        *data++ = gdTrueColorGetRed(px);
+		        const int px = gdImageTrueColorPixel(im, x, y);
+		        *data++ = (unsigned char)gdTrueColorGetBlue(px);
+		        *data++ = (unsigned char)gdTrueColorGetGreen(px);
+		        *data++ = (unsigned char)gdTrueColorGetRed(px);
 		        *data++ = 0xFF;
 		    }
 		}
@@ -165,29 +152,32 @@ static void gd_loadimage_cairo(GVJ_t * job, usershape_t *us, boxf b, boolean fil
 	else {
 	    for (y = 0; y < height; y++) {
 		for (x = 0; x < width; x++) {
-		    px = gdImagePalettePixel(im, x, y);
-		    *data++ = im->blue[px];
-		    *data++ = im->green[px];
-		    *data++ = im->red[px];
-		    *data++ = (px==im->transparent)?0x00:0xff;
+		    const int px = gdImagePalettePixel(im, x, y);
+		    *data++ = (unsigned char)im->blue[px];
+		    *data++ = (unsigned char)im->green[px];
+		    *data++ = (unsigned char)im->red[px];
+		    *data++ = px == im->transparent ? 0x00 : 0xff;
 		}
 	    }
 	}
 
         cairo_save(cr);
         cairo_translate(cr, b.LL.x, -b.UR.y);
-        cairo_scale(cr, (b.UR.x - b.LL.x)/(us->w), (b.UR.y - b.LL.y)/(us->h));
+        cairo_scale(cr, (b.UR.x - b.LL.x) / us->w, (b.UR.y - b.LL.y) / us->h);
         cairo_set_source_surface (cr, surface, 0, 0);
         cairo_paint (cr);
         cairo_restore(cr);
 
 	cairo_surface_destroy(surface);
+	free(data);
     }
 }
 #endif
 
-static void gd_loadimage_ps(GVJ_t * job, usershape_t *us, boxf b, boolean filled)
+static void gd_loadimage_ps(GVJ_t * job, usershape_t *us, boxf b, bool filled)
 {
+    (void)filled;
+
     gdImagePtr im = NULL;
     int X, Y, x, y, px;
 
@@ -231,13 +221,13 @@ static void gd_loadimage_ps(GVJ_t * job, usershape_t *us, boxf b, boolean filled
 
         /* this sets the position of the image */
         gvprintf(job, "%g %g translate\n",
-		(b.LL.x + (b.UR.x - b.LL.x) * (1. - (job->dpi.x) / 96.) / 2.),
-	        (b.LL.y + (b.UR.y - b.LL.y) * (1. - (job->dpi.y) / 96.) / 2.));
+		b.LL.x + (b.UR.x - b.LL.x) * (1. - (job->dpi.x) / 96.) / 2.,
+	        b.LL.y + (b.UR.y - b.LL.y) * (1. - (job->dpi.y) / 96.) / 2.);
 
         /* this sets the rendered size to fit the box */
         gvprintf(job,"%g %g scale\n",
-		((b.UR.x - b.LL.x) * (job->dpi.x) / 96.),
-		((b.UR.y - b.LL.y) * (job->dpi.y) / 96.));
+		(b.UR.x - b.LL.x) * (job->dpi.x) / 96.,
+		(b.UR.y - b.LL.y) * (job->dpi.y) / 96.);
     
         /* xsize ysize bits-per-sample [matrix] */
         gvprintf(job, "%d %d 8 [%d 0 0 %d 0 %d]\n", X, Y, X, -Y, Y);
