@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -205,22 +205,21 @@ static void quartzgen_begin_page(GVJ_t * job)
 			if (temp_file_descriptor >= 0
 			    && ftruncate(temp_file_descriptor,
 					 buffer_size) == 0) {
-			    buffer =
-				mmap(NULL, buffer_size,
-				     PROT_READ | PROT_WRITE,
-				     MAP_FILE | MAP_SHARED,
-				     temp_file_descriptor, 0);
+			    buffer = mmap(NULL, buffer_size, PROT_READ | PROT_WRITE,
+			                  MAP_FILE | MAP_PRIVATE, temp_file_descriptor, 0);
 			    if (buffer == MAP_FAILED)
 				buffer = NULL;
 			}
 			fclose(temp_file);
 		    }
 		}
-		if (!buffer)
-		    buffer = mmap(NULL,
-				  buffer_size,
-				  PROT_READ | PROT_WRITE,
-				  MAP_ANON | MAP_SHARED, -1, 0);
+		if (buffer == NULL) {
+		    buffer = mmap(NULL, buffer_size, PROT_READ | PROT_WRITE,
+				  MAP_ANON | MAP_PRIVATE, -1, 0);
+		    if (buffer == MAP_FAILED) {
+			buffer = NULL;
+		    }
+		}
 #endif
 #endif
 
@@ -392,13 +391,11 @@ static void quartzgen_ellipse(GVJ_t * job, pointf * A, int filled)
     quartzgen_path(job, filled);
 }
 
-static void quartzgen_polygon(GVJ_t * job, pointf * A, int n, int filled)
-{
+static void quartzgen_polygon(GVJ_t *job, pointf *A, size_t n, int filled) {
     /* convert polygon into the current path */
     CGContextRef context = job->context;
     CGContextMoveToPoint(context, A[0].x, A[0].y);
-    int i;
-    for (i = 1; i < n; ++i)
+    for (size_t i = 1; i < n; ++i)
 	CGContextAddLineToPoint(context, A[i].x, A[i].y);
     CGContextClosePath(context);
 
@@ -406,18 +403,11 @@ static void quartzgen_polygon(GVJ_t * job, pointf * A, int n, int filled)
     quartzgen_path(job, filled);
 }
 
-static void
-quartzgen_bezier(GVJ_t * job, pointf * A, int n, int arrow_at_start,
-		 int arrow_at_end, int filled)
-{
-    (void)arrow_at_start;
-    (void)arrow_at_end;
-
+static void quartzgen_bezier(GVJ_t *job, pointf *A, size_t n, int filled) {
     /* convert bezier into the current path */
     CGContextRef context = job->context;
     CGContextMoveToPoint(context, A[0].x, A[0].y);
-    int i;
-    for (i = 1; i < n; i += 3)
+    for (size_t i = 1; i < n; i += 3)
 	CGContextAddCurveToPoint(context, A[i].x, A[i].y, A[i + 1].x,
 				 A[i + 1].y, A[i + 2].x, A[i + 2].y);
 
@@ -425,17 +415,15 @@ quartzgen_bezier(GVJ_t * job, pointf * A, int n, int arrow_at_start,
     quartzgen_path(job, filled);
 }
 
-static void quartzgen_polyline(GVJ_t * job, pointf * A, int n)
-{
+static void quartzgen_polyline(GVJ_t *job, pointf *A, size_t n) {
     /* convert polyline into the current path */
     CGContextRef context = job->context;
     CGContextMoveToPoint(context, A[0].x, A[0].y);
-    int i;
-    for (i = 1; i < n; ++i)
+    for (size_t i = 1; i < n; ++i)
 	CGContextAddLineToPoint(context, A[i].x, A[i].y);
 
     /* draw the ellipse */
-    quartzgen_path(job, FALSE);
+    quartzgen_path(job, 0);
 }
 
 static gvrender_engine_t quartzgen_engine = {
