@@ -1,6 +1,6 @@
 #include <string_view>
 
-#include <catch2/catch.hpp>
+#include <catch2/catch_all.hpp>
 #include <fmt/format.h>
 
 #include "svg_analyzer.h"
@@ -11,8 +11,8 @@ TEST_CASE("Node penwidth",
           "'stroke-width' attribute correctly for nodes in the generated SVG") {
 
   const auto shape = GENERATE(filter(
-      [](std::string_view shape) {
-        return !node_shapes_without_svg_shape.contains(shape);
+      [](std::string_view s) {
+        return !node_shapes_without_svg_shape.contains(s);
       },
       from_range(all_node_shapes)));
   INFO("Shape: " << shape);
